#       
#           AutoDock | Raccoon2
#
#       Copyright 2013, Stefano Forli
#          Molecular Graphics Lab
#  
#     The Scripps Research Institute 
#           _  
#          (,)  T  h e
#         _/
#        (.)    S  c r i p p s
#          \_
#          (,)  R  e s e a r c h
#         ./  
#        ( )    I  n s t i t u t e
#         '
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import os, sys
from CADD.Raccoon2 import HelperFunctionsN3P as hf
from DebugTools import DebugObj
import threading
from datetime import datetime

class Library(DebugObj):
    """ contains and maps all the library data
    
        allows to get the list of ligands, generates
        the files for the jobs submission, does the 
        filtering on the fly...

    """

    def __init__(self, server, name = 'noname', _format = 'pdbqt', info = None, debug=False):
        DebugObj.__init__(self, debug)
        self.dprint("init class")
        self.server = server
        self.items = {}
        # NOTE this variable is set automatically 
        # by self.upload() and self._readIndex()
        self.path = None

        self.options = { 'split' : 1000, # create sub-directories each N ligs
                    }

        self.ready = False
        self.info = { 'name'   : name,
                      'count'  : 0,
                      'date'   : None,
                      'format' : _format,
                      'index_file': None,
                      'comments': "",
                      'properties' : {},
                      'type' : None,
                    }

        self.filters = {}

        if info:
            self.info = info
            # restore the library obj from an index file
            if self.info['index_file']:
                self._readIndex()


    def updatestatus(self):
        """set the .ready attribute"""
        c = (self.info['count'] > 0)
        i = (not self.info['index_file'] == None)
        self.dprint("count[%s] index[%s]" % (c, i))
        self.ready = (c and i)

    def name(self):
        """ used for sorting libraries """
        return self.info['name']

    # XXX potentially useless
    def setInfo(self, info):
        """set the library info as the dictionary provided"""
        self.info = info


    def getInfo(self, key=None):
        """retrieve all info or a specific one about the library"""
        if key == None:
            return self.info
        else:   
            return self.info[key]

    def _date(self):
        """generate date format for library info"""
        d = datetime.now()
        return [d.year, d.month, d.day, d.hour, d.minute, d.second ]

    def filterItems(self, items = [], filterset = {}):
        """return items passing filters

            a list of items can be optonally passed

            filterset is optional if self.filters is defined

         filterset = { property1 : [min, max],  ....}
        """
        if items == []:
            items == self.items.keys()
        if filterset == {}:
            filterset = self.filters

        filtered = []
        for i in items:
            accepted = True
            iprop = self.items[i]['properties']
            for f in filterset.keys():
                fmin, fmax = filterset[f]
                if not (iprop[f] >= fmin) or (iprop[f] <= fmax):
                    accepted = False
                    break
            if accepted:
                filtered.append(i)
        return filtered

    def setfilters(self, filterset = {}, dofilter=False):
        """ set the filter set to the required filterset"""
        self.filters = filterset
        if self.dofilter:
            return self.filterItems()

    def delfilters(self):
        """ remove all filters"""
        self.filters = {}

    def _checkitem(self, item, dup = 'rename'):
        """ check if item is already registered and 
            provide fallback options for the naming
            scheme:

                dup = 'overwrite'  duplicates not allowed (last kept)
                dup = 'rename'     duplicates renamed
                dup = 'skip'       duplicates not allowed (first kept)
        """
        name,ext = os.path.splitext(item)
        name = os.path.basename(name)
        # manage ligand name
        if self.items.get(name):
            self.dprint("ligand with same name found")
            if dup == 'rename':
                self.dprint("duplicates will be RENAMED")
                new_name = name
                count = 0
                while new_name in self.items.keys():
                    count += 1
                    new_name = new_name+"_%d" % count
                name = new_name
            elif dup == 'overwrite':
                self.dprint("duplicates will be OVEWRITTEN")
                pass
            elif dup == 'skip':
                self.dprint("duplicates will be SKIPPED")
                return False
        else:
            self.dprint("new item")
        destname = name+ext
        return name, destname

    def _additems(self, items, propfunc=None, dup = 'rename', stopcheck=None, showpercent=None, GUI=None):
        """ 
            add [items] to the current item list

            propfunc    functio to be used to calculate properties

            dup = 'overwrite'  duplicates not allowed (last kept)
            dup = 'rename'     duplicates renamed
            dup = 'skip'       duplicates not allowed (first kept)
        """
        self.dprint("adding [%d] items"% len(items), new=1)
        problematic = []
        c = 1
        # XXX FIXME profile here!
        for i in items:
            #source = i
            checkreport = self._checkitem(i, dup = dup)
            if checkreport:
                name, destname = checkreport
            #if self._checkitem(i):
                # manage ligand properties
                if propfunc:
                    report, prop = propfunc(i)
                else:
                    report, prop = True, {}
                if report:
                    self.items[name] = { 'source'    : i,
                                         'fullpath'  : destname, 
                                         # destname can be updated with the suffix dir for splitting
                                         'properties': prop,
                                       }
                else:
                    self.dprint("problematic item [%s] : [%s]" % (i, prop))
                    problematic.append([ i, prop ])
            c+=1
            if not stopcheck==None:
                if stopcheck():
                    self.dprint("stopcheck() requested stopping...")
                    break
            if not GUI == None:
                GUI.update()
            if not showpercent == None:
                pc = hf.percent(c, len(items) )
                showpercent(pc)
        self.updatestatus()
        return problematic

    def _delitems(self, names=[]):
        """delete items by name 
        
            THIS FUNCTION SHOULD NOT BE USED ON ITS OWN:
            THE CHILD SHOULD WRAP IT IN ITS METHOD
        """
        self.dprint("deleting [%d] items" % len(names))
        deleted = [ self.items.pop(n) for n in names ] 
        self.info['count'] =  len(self.items)
        self.updatestatus()
        return deleted


    def _writeIndex(self, fname=None):
        """build the zlib-compressed JSON index with all per-ligand info
           about the library file to fname (default is library.db)

           child objects have to implement their self.saveIndex method
           that will call this and perform any other operations
        """
        if fname == None:
            # build default with path
            if self.path == None:
                self.dprint('fname and self.path not defined, returning')
                return False
            fname = self.path + '/' + 'library.db'
        self.dprint('index file [%s]' % fname)
        self.info['index_file'] = fname
        self.updatestatus()
        #return self.server.writeJson(fname, data=self.items, compression=True)
        return self.server.writeMarshal(fname, data=self.items, compression=True)

    def _readIndex(self, fname=None):
        """
           rebuild the zlib-compressed library info from the index file
           child objects have to implement their self.loadIndex() method
           that will call this and perform any other operations
        """
        if fname == None:
            fname = self.info['index_file']
        if fname == None:
            self.dprint('index filename not defined')
            return False
        self.dprint('reading fname[%s]' % fname)
        self.path = fname.rsplit('/',1)[0]
        self.dprint('set lib path to [%s]' % self.path)
        try:
            #r = self.server.readJson(fname, compression=True)
            r = self.server.readMarshal(fname, compression=True)
        except:
            self.dprint("error in reading index file [%s] : %s" % (fname, sys.exc_info()[1]) )
            return False
        if r: 
            self.items = r
            self.dprint('data read successfully')
            #self.info['index_file'] = fname
        self.updatestatus()
        return True

    def register(self, overwrite=False):
        """ register itself in the master_index_file on the server
        """
        self.dprint("registration requested")
        if not self.ready == True:
            self.dprint('library not ready, stopping registration.')
            return False
        serverlib = self.server.properties['libraries']
        name = self.info['name']
        if self.server.getLibrary(name):
            self.dprint("name conflict found [%s]" % name)
            if not overwrite:
                self.dprint("no overwrite, RETURNING")
                return False
            else:
                self.dprint("overwriting requested")
                self.server.delLibrary(name)
        self.server.properties['libraries'].append(self)
        self.server.saveLibraryIndex()
        return True

    def unregister(self, delfiles=True):
        """remove itself from master_index_file"""
        name = self.info['name']
        self.dprint("unregistering library [%s]" % name)
        return self.server.delLibrary( name, delfiles)

    def upload(self, dirname=None, destpath=None, autoregister=False, overwrite=False, bg=False):
        """upload the library to the remote server $dest path"""
        if dirname == None:
            dirname = hf.validFilename(self.name())
        if dirname == '':
            dirname = 'libdir'
            self.dprint("warning: lib dirname empty, providing substutute [%s]" % libdir)
        if destpath == None:
            destpath == self.server.getLibraryPath()
        if (self.server == None) or (self.server == 'localhost'):
            self._localcopy( dirname, destpath, autoregister, overwrite, bg)
        else:
            self._remotecopy( dirname, destpath, autoregister, overwrite, bg)
        self.updatestatus()
            

    def _localcopy(self, dirname, destpath, autoregister=False, overwrite=False, bg=False):
        """ perform the copy on local path"""
        # XXX LOCAL XXX TODO
        # generating fullpath
        fulldirname = destpath+os.sep+dirname 
        print "I SHOULD COPY HERE", filldirname
        # check if exist
        #   check what's overwrite
        # check splitting settigs
        #   update name path
        # perform the copy
        # autoregister

    def _remotecopy(self, dirname, destpath, autoregister=False, overwrite=False, bg=True):
        """ perform the copy on remote path"""
        # generating fullpath destination dir
        self.path = destpath + '/' + dirname
        if self.server.ssh.exists(self.path):
            self.dprint('remote library dirname[%s] exist in the path[%s] => [%s]...' % (dirname, destpath, self.path))
            if not overwrite:
                self.dprint('overwrite not allowed, returning')
                return
            else:
                self.server.ssh.remove([self.path])
        # update the ligand destination with split_path
        step = self.options['split']
        items = sorted(self.items.keys())
        total = len(items)
        if (not step == None) and (step <= total):
            count = 0
            for i in items:
                self.dprint("updating splitted name [%s] =>" % self.items[i]['fullpath']),
                self.items[i]['fullpath'] = '/'.join([hf.splitdir(count, total, step),
                            self.items[i]['fullpath']])
                self.dprint("[%s]" % self.items[i]['fullpath'])
                count+=1
        transfer_list = []
        for i in items:
            dest = '/'.join([dirname, self.items[i]['fullpath'] ])
            transfer_list.append( [self.items[i]['source'], dest])
        # transfer files
        mule = self.server.transfer
        self.dprint("starting the transfer with the mule[%s]" % mule)
        mule.upload(files = transfer_list, remotedestpath=destpath, bg=bg)
        self.dprint("mule started... ongoing...")

        if autoregister:
            if not bg:
                libindex = self.path + '/' + 'library.db'
                self.saveIndex(libindex)
                self.registerLibrary(overwrite)
            else:
                print "AUTOREGISTER DISABLED! BACKGROUND UPLOAD (TO FIX)"


    def getFilesPath(self):
        """ return path where files are saved """
        if (self.server == None) or (self.server == 'localhost'):
            return os.path.dirname(self.info['index_file'])
        else:
            return self.info['index_file'].rsplit('/', 1)[0]




    def getItems(self, names=None):
        """ retrieve fullpath filenames for requested
            ligands (i.e. list of filtered files)
        """
        if names == None:
            names = self.items.keys()
        names = self.filterItems(items=names)
        libpath = self.getFilesPath()
        if (self.server == None) or (self.server == 'localhost'):
            sep = os.sep
        else:
            sep = '/'
        local_path_list = [ self.items[n]['fullpath'] for n in names ]
        full_path_list = [ libpath + sep + f for f in local_path_list ]
        return full_path_list


class LigandLibrary(Library):
    """
    """

    def __init__(self, server, name = 'liglibrary', _format='pdbqt', info=None, debug = False):
        Library.__init__(self, server, name, _format, info, debug)
        self.info['type'] = 'ligand'
        self._refreshProperties()


    def addLigands(self, ligands=[], dup='rename', stopcheck=None, showpercent=None, GUI=None): #, donevar=None, resultvar=None):
        """ 
            add [ligands] to the current list of ligands

            ligands = list_of_ligand_files

            dup = 'overwrite'  duplicates not allowed (last kept)
            dup = 'rename'     duplicates renamed
            dup = 'skip'       duplicates not allowed (first kept)
        """
        problematic = self._additems(items=ligands, propfunc=self._getligprop, dup=dup, 
            stopcheck=stopcheck, showpercent=showpercent, GUI=GUI)
        self.info['date'] = self._date()
        self._refreshProperties()
        #print "DEBUG Writing json here"
        #hf.writejson('_debug_LibItems.log', self.items)
        return problematic


    def deleteLigands(self, names):
        """delete ligands by name"""
        deleted = self._delitems(names=names)
        self._refreshProperties()
        return deleted

    def loadIndex(self, fname=None):
        """ rebuild the library obj info from the 
            saved index file (def: library.db)
        """
        self._readIndex(fname)
        self._refreshProperties()

    def saveIndex(self, fname=None):
        """ save the info about this obj 
            in the index file (def: library.db)
        """
        self._writeIndex(fname)


    def _refreshProperties(self):
        """update/initialize ligand library properties 
        
        
            TODO: create bins for properties distribution
                  similar to what zinc does
                http://zinc.docking.org/catalogs/asin
        """
        self.dprint("refreshing library properties...")
        lib_prop = {} 

        items = self.items.keys()
        self.info['count'] = len(items)
        for p in  ['heavy', 'mw', 'hba', 'hbd', 'tors']:
            values = sorted([ self.items[i]['properties'][p] for i in items ])
            try:    lib_prop[p] = [ values[0], values[-1] ]
            except: lib_prop[p] = []
        lib_prop['atypes'] = set([])
        for i in items:
            lib_prop['atypes'] = lib_prop['atypes'].union( set(self.items[i]['properties']['atypes']) )
        lib_prop['atypes'] = list(lib_prop['atypes'])

        self.info['properties'] = lib_prop
        self.dprint(" DONE ")

    def _getligprop(self, filename):
        """ calculate basic properties from PDBQT ligand
            mw, hba, hbd, heavy atoms
        """
        try:
            lines = hf.getLines(filename)
            atoms = hf.getAtoms(lines)
        except:
            return False, sys.exc_info()[1]
        if len(lines) == 0:
            return False, "empty file"
        if len(atoms) == 0:
            return False, "no atoms (INVALID FILE)"
        try:
            tors = int(lines[-1].split()[1])
        except:
            return False, "no torsions (INVALID FILE)"
        # hb
        hba, hbd = [ len(x) for x in  hf.findHbAccepDon(atoms) ]
        # atypes ( report all types with duplicates, to be used for updating lib types)
        atypes = [ x.split()[-1] for x in atoms ]
        # heavy
        heavy = len([ x for x in atypes if not x == 'HD' ])
        # mw
        mw = 0
        for a in atypes:
            mw += hf.adtypes.get( a, [None,0])[1]
        atypes = list(set(atypes)) #[ x.split()[-1] for x in atoms ]))
        return True, {'tors' : tors, 'heavy' : heavy, 'mw' : mw, 'hba' : hba, 'hbd': hbd, 'atypes': atypes }

    

class ReceptorLibrary(Library):
    """ """
    def __init__(self):
        pass

        # XXX it can provide a collection of receptor-flexres-config triplets
        # user should be able to choose if it wants to use them or not
        # provide auto-Config for blind docking? 


knownlibraries = { 'ligand' : LigandLibrary , 
                   'receptor' : ReceptorLibrary }



#!/bin/ksh ~/.mgltools/pythonsh
########################################################################
#
#    Vision Network - Python source code - file generated by vision
#    Wednesday 18 May 2011 14:03:30 
#    
#       The Scripps Research Institute (TSRI)
#       Molecular Graphics Lab
#       La Jolla, CA 92037, USA
#
# Copyright: Daniel Stoffler, Michel Sanner and TSRI
#   
# revision: Guillaume Vareille
#  
#########################################################################
#
# $Header: /opt/cvs/CADD/workflows/MDanalysis/AMDNBCR_0.2_net.py,v 1.2 2011/06/29 01:45:14 jren Exp $
#
# $Id: AMDNBCR_0.2_net.py,v 1.2 2011/06/29 01:45:14 jren Exp $
#


if __name__=='__main__':
    from sys import argv
    if '--help' in argv or '-h' in argv or '-w' in argv: # run without Vision
        withoutVision = True
        from Vision.VPE import NoGuiExec
        ed = NoGuiExec()
        from NetworkEditor.net import Network
        import os
        masterNet = Network("process-"+str(os.getpid()))
        ed.addNetwork(masterNet)
    else: # run as a stand alone application while vision is hidden
        withoutVision = False
        from Vision import launchVisionToRunNetworkAsApplication, mainLoopVisionToRunNetworkAsApplication
	if '-noSplash' in argv:
	    splash = False
	else:
	    splash = True
        masterNet = launchVisionToRunNetworkAsApplication(splash=splash)
        import os
        masterNet.filename = os.path.abspath(__file__)
from traceback import print_exc
## loading libraries ##
from WebServices.VisionInterface.WSNodes import wslib
from Vision.StandardNodes import stdlib
try:
    masterNet
except (NameError, AttributeError): # we run the network outside Vision
    from NetworkEditor.net import Network
    masterNet = Network()

masterNet.getEditor().addLibraryInstance(wslib,"WebServices.VisionInterface.WSNodes", "wslib")

masterNet.getEditor().addLibraryInstance(stdlib,"Vision.StandardNodes", "stdlib")

from WebServices.VisionInterface.WSNodes import addOpalServerAsCategory
try:
    addOpalServerAsCategory("http://rocce.ucsd.edu/opal2", replace=False)
except:
    pass
try:
    ## saving node Input Directory Browser ##
    from Vision.StandardNodes import DirBrowserNE
    Input_Directory_Browser_0 = DirBrowserNE(constrkw={}, name='Input Directory Browser', library=stdlib)
    masterNet.addNode(Input_Directory_Browser_0,375,8)
    Input_Directory_Browser_0.inputPortByName['directory'].widget.set(r"AMDNBCR_0.2_input", run=False)
    apply(Input_Directory_Browser_0.configure, (), {'paramPanelImmediate': 1})
except:
    print "WARNING: failed to restore DirBrowserNE named Input Directory Browser in network masterNet"
    print_exc()
    Input_Directory_Browser_0=None

try:
    ## saving node NumProcessors ##
    from Vision.StandardNodes import EntryNE
    NumProcessors_1 = EntryNE(constrkw={}, name='NumProcessors', library=stdlib)
    masterNet.addNode(NumProcessors_1,1043,12)
    NumProcessors_1.inputPortByName['entry'].widget.set(r"4", run=False)
    code = """def doit(self, entry):
    if len(str(entry))!=0:
        self.outputData(string=entry)

"""
    NumProcessors_1.configure(function=code)
    apply(NumProcessors_1.configure, (), {'paramPanelImmediate': 1})
except:
    print "WARNING: failed to restore EntryNE named NumProcessors in network masterNet"
    print_exc()
    NumProcessors_1=None

try:
    ## saving node Output Directory Browser ##
    from Vision.StandardNodes import DirBrowserNE
    Output_Directory_Browser_2 = DirBrowserNE(constrkw={}, name='Output Directory Browser', library=stdlib)
    masterNet.addNode(Output_Directory_Browser_2,715,9)
    Output_Directory_Browser_2.inputPortByName['directory'].widget.set(r"AMDNBCR_0.2_output", run=False)
    apply(Output_Directory_Browser_2.configure, (), {'paramPanelImmediate': 1})
except:
    print "WARNING: failed to restore DirBrowserNE named Output Directory Browser in network masterNet"
    print_exc()
    Output_Directory_Browser_2=None

try:
    ## saving node Config File Browser ##
    from Vision.StandardNodes import FileBrowserNE
    Config_File_Browser_3 = FileBrowserNE(constrkw={}, name='Config File Browser', library=stdlib)
    masterNet.addNode(Config_File_Browser_3,67,22)
    Config_File_Browser_3.inputPortByName['filename'].widget.set(r"AMDNBCR_0.2_input/sample-aMD.conf", run=False)
    apply(Config_File_Browser_3.configure, (), {'paramPanelImmediate': 1})
except:
    print "WARNING: failed to restore FileBrowserNE named Config File Browser in network masterNet"
    print_exc()
    Config_File_Browser_3=None

try:
    ## saving node AMDws ##
    from NetworkEditor.macros import MacroNode
    AMDws_4 = MacroNode(name='AMDws')
    masterNet.addNode(AMDws_4, 357, 184)
    output_Ports_6 = AMDws_4.macroNetwork.opNode
    apply(output_Ports_6.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
    output_Ports_6.move(182, 394)
    from NetworkEditor.items import FunctionNode
    NAMD_rocce_ucsd_edu_7 = FunctionNode(functionOrString='NAMD_rocce_ucsd_edu', host="http://rocce.ucsd.edu/opal2", namedArgs={'tarball': '', 'namdarg': '', 'localRun': False, 'execPath': '', 'procs': ''}, constrkw={'functionOrString': "'NAMD_rocce_ucsd_edu'", 'host': '"http://rocce.ucsd.edu/opal2"', 'namedArgs': {'tarball': '', 'namdarg': '', 'localRun': False, 'execPath': '', 'procs': ''}}, name='NAMD_rocce_ucsd_edu', library=wslib)
    AMDws_4.macroNetwork.addNode(NAMD_rocce_ucsd_edu_7,198,293)
    apply(NAMD_rocce_ucsd_edu_7.inputPortByName['tarball'].configure, (), {'defaultValue': None})
    apply(NAMD_rocce_ucsd_edu_7.inputPortByName['namdarg'].configure, (), {'defaultValue': None})
    apply(NAMD_rocce_ucsd_edu_7.inputPortByName['procs'].configure, (), {'defaultValue': None})
    NAMD_rocce_ucsd_edu_7.inputPortByName['tarball'].rebindWidget()
    NAMD_rocce_ucsd_edu_7.inputPortByName['tarball'].widget.set(r"", run=False)
    NAMD_rocce_ucsd_edu_7.inputPortByName['tarball'].unbindWidget()
    NAMD_rocce_ucsd_edu_7.inputPortByName['namdarg'].rebindWidget()
    NAMD_rocce_ucsd_edu_7.inputPortByName['namdarg'].widget.set(r"", run=False)
    NAMD_rocce_ucsd_edu_7.inputPortByName['namdarg'].unbindWidget()
    NAMD_rocce_ucsd_edu_7.inputPortByName['localRun'].widget.set(0, run=False)
    NAMD_rocce_ucsd_edu_7.inputPortByName['execPath'].widget.set(r"", run=False)
    NAMD_rocce_ucsd_edu_7.inputPortByName['procs'].rebindWidget()
    NAMD_rocce_ucsd_edu_7.inputPortByName['procs'].widget.set(r"", run=False)
    NAMD_rocce_ucsd_edu_7.inputPortByName['procs'].unbindWidget()
    apply(NAMD_rocce_ucsd_edu_7.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
    from Vision.StandardNodes import MakeZipFileNE
    Make_Zip_File_8 = MakeZipFileNE(constrkw={}, name='Make Zip File', library=stdlib)
    AMDws_4.macroNetwork.addNode(Make_Zip_File_8,44,145)
    apply(Make_Zip_File_8.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
    from Vision.StandardNodes import Eval
    getConfigName_9 = Eval(constrkw={}, name='getConfigName', library=stdlib)
    AMDws_4.macroNetwork.addNode(getConfigName_9,217,172)
    getConfigName_9.inputPortByName['command'].widget.set(r"os.path.basename(in1)", run=False)
    getConfigName_9.inputPortByName['importString'].widget.set(r"", run=False)
    apply(getConfigName_9.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
    from Vision.StandardNodes import Eval
    prepareForZip_10 = Eval(constrkw={}, name='prepareForZip', library=stdlib)
    AMDws_4.macroNetwork.addNode(prepareForZip_10,143,91)
    prepareForZip_10.inputPortByName['command'].widget.set(r"os.path.basename(in1)", run=False)
    prepareForZip_10.inputPortByName['importString'].widget.set(r"", run=False)
    apply(prepareForZip_10.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
    from WebServices.VisionInterface.WSNodes import GetMainURLFromListNode
    GetMainURLFromList_11 = GetMainURLFromListNode(constrkw={}, name='GetMainURLFromList', library=wslib)
    AMDws_4.macroNetwork.addNode(GetMainURLFromList_11,199,342)
    apply(GetMainURLFromList_11.configure, (), {'paramPanelImmediate': 1, 'expanded': False})

    ## saving connections for network AMDws ##
    AMDws_4.macroNetwork.freeze()
    input_Ports_5 = AMDws_4.macroNetwork.ipNode
    if input_Ports_5 is not None and getConfigName_9 is not None:
        try:
            AMDws_4.macroNetwork.connectNodes(
                input_Ports_5, getConfigName_9, "new", "in1", blocking=True
                , splitratio=[0.56731089905051557, 0.69034575704309797])
        except:
            print "WARNING: failed to restore connection between input_Ports_5 and getConfigName_9 in network AMDws_4.macroNetwork"
    if input_Ports_5 is not None and prepareForZip_10 is not None:
        try:
            AMDws_4.macroNetwork.connectNodes(
                input_Ports_5, prepareForZip_10, "new", "in1", blocking=True
                , splitratio=[0.50135048628782752, 0.38016101490352128])
        except:
            print "WARNING: failed to restore connection between input_Ports_5 and prepareForZip_10 in network AMDws_4.macroNetwork"
    if input_Ports_5 is not None and Make_Zip_File_8 is not None:
        try:
            AMDws_4.macroNetwork.connectNodes(
                input_Ports_5, Make_Zip_File_8, "prepareForZip_in1", "input_directory", blocking=True
                , splitratio=[0.26026655091979006, 0.38836623894413969])
        except:
            print "WARNING: failed to restore connection between input_Ports_5 and Make_Zip_File_8 in network AMDws_4.macroNetwork"
    if prepareForZip_10 is not None and Make_Zip_File_8 is not None:
        try:
            AMDws_4.macroNetwork.connectNodes(
                prepareForZip_10, Make_Zip_File_8, "result", "output_name", blocking=True
                , splitratio=[0.54034866876508625, 0.61013349942874484])
        except:
            print "WARNING: failed to restore connection between prepareForZip_10 and Make_Zip_File_8 in network AMDws_4.macroNetwork"
    if Make_Zip_File_8 is not None and NAMD_rocce_ucsd_edu_7 is not None:
        try:
            AMDws_4.macroNetwork.connectNodes(
                Make_Zip_File_8, NAMD_rocce_ucsd_edu_7, "zipfile", "tarball", blocking=True
                , splitratio=[0.25421834768677798, 0.72322220314479146])
        except:
            print "WARNING: failed to restore connection between Make_Zip_File_8 and NAMD_rocce_ucsd_edu_7 in network AMDws_4.macroNetwork"
    if getConfigName_9 is not None and NAMD_rocce_ucsd_edu_7 is not None:
        try:
            AMDws_4.macroNetwork.connectNodes(
                getConfigName_9, NAMD_rocce_ucsd_edu_7, "result", "namdarg", blocking=True
                , splitratio=[0.29539778940348749, 0.48121221099858358])
        except:
            print "WARNING: failed to restore connection between getConfigName_9 and NAMD_rocce_ucsd_edu_7 in network AMDws_4.macroNetwork"
    if input_Ports_5 is not None and NAMD_rocce_ucsd_edu_7 is not None:
        try:
            AMDws_4.macroNetwork.connectNodes(
                input_Ports_5, NAMD_rocce_ucsd_edu_7, "new", "procs", blocking=True
                , splitratio=[0.73370540958251707, 0.40716190001312769])
        except:
            print "WARNING: failed to restore connection between input_Ports_5 and NAMD_rocce_ucsd_edu_7 in network AMDws_4.macroNetwork"
    if NAMD_rocce_ucsd_edu_7 is not None and GetMainURLFromList_11 is not None:
        try:
            AMDws_4.macroNetwork.connectNodes(
                NAMD_rocce_ucsd_edu_7, GetMainURLFromList_11, "result", "urls", blocking=True
                , splitratio=[0.62509867822831222, 0.37094200197260763])
        except:
            print "WARNING: failed to restore connection between NAMD_rocce_ucsd_edu_7 and GetMainURLFromList_11 in network AMDws_4.macroNetwork"
    output_Ports_6 = AMDws_4.macroNetwork.opNode
    if GetMainURLFromList_11 is not None and output_Ports_6 is not None:
        try:
            AMDws_4.macroNetwork.connectNodes(
                GetMainURLFromList_11, output_Ports_6, "newurl", "new", blocking=True
                , splitratio=[0.66338768468729703, 0.52542303488924691])
        except:
            print "WARNING: failed to restore connection between GetMainURLFromList_11 and output_Ports_6 in network AMDws_4.macroNetwork"
    AMDws_4.macroNetwork.unfreeze()

    ## modifying MacroInputNode dynamic ports
    input_Ports_5.outputPorts[1].configure(name='eval_in1')
    input_Ports_5.outputPorts[2].configure(name='prepareForZip_in1')
    input_Ports_5.outputPorts[3].configure(name='NAMD_rocce_ucsd_edu_procs')

    ## modifying MacroOutputNode dynamic ports
    output_Ports_6.inputPorts[1].configure(singleConnection='auto')
    output_Ports_6.inputPorts[1].configure(name='GetMainURLFromList_newurl')
    AMDws_4.inputPorts[0].configure(name='eval_in1')
    AMDws_4.inputPorts[0].configure(datatype='string')
    AMDws_4.inputPorts[1].configure(name='prepareForZip_in1')
    AMDws_4.inputPorts[1].configure(datatype='string')
    AMDws_4.inputPorts[2].configure(name='NAMD_rocce_ucsd_edu_procs')
    AMDws_4.inputPorts[2].configure(datatype='string')
    ## configure MacroNode input ports
    AMDws_4.outputPorts[0].configure(name='GetMainURLFromList_newurl')
    AMDws_4.outputPorts[0].configure(datatype='string')
    ## configure MacroNode output ports
    AMDws_4.shrink()
    apply(AMDws_4.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
except:
    print "WARNING: failed to restore MacroNode named AMDws in network masterNet"
    print_exc()
    AMDws_4=None

try:
    ## saving node DownloadDir ##
    from WebServices.VisionInterface.WSNodes import DownloadSaveDirNode
    DownloadDir_12 = DownloadSaveDirNode(constrkw={}, name='DownloadDir', library=wslib)
    masterNet.addNode(DownloadDir_12,494,320)
    apply(DownloadDir_12.inputPortByName['url'].configure, (), {'defaultValue': None})
    DownloadDir_12.inputPortByName['url'].rebindWidget()
    DownloadDir_12.inputPortByName['url'].widget.set(r"", run=False)
    DownloadDir_12.inputPortByName['url'].unbindWidget()
    apply(DownloadDir_12.configure, (), {'paramPanelImmediate': 1})
except:
    print "WARNING: failed to restore DownloadSaveDirNode named DownloadDir in network masterNet"
    print_exc()
    DownloadDir_12=None

#masterNet.run()
masterNet.freeze()

## saving connections for network AMDNBCR-0.2 ##
AMDws_4 = masterNet.nodes[4]
if Config_File_Browser_3 is not None and AMDws_4 is not None:
    try:
        masterNet.connectNodes(
            Config_File_Browser_3, AMDws_4, "filename", "eval_in1", blocking=True
            , splitratio=[0.65150653312544904, 0.31836383152848224])
    except:
        print "WARNING: failed to restore connection between Config_File_Browser_3 and AMDws_4 in network masterNet"
if Input_Directory_Browser_0 is not None and AMDws_4 is not None:
    try:
        masterNet.connectNodes(
            Input_Directory_Browser_0, AMDws_4, "directory", "prepareForZip_in1", blocking=True
            , splitratio=[0.46424478122839202, 0.70216128328369165])
    except:
        print "WARNING: failed to restore connection between Input_Directory_Browser_0 and AMDws_4 in network masterNet"
if NumProcessors_1 is not None and AMDws_4 is not None:
    try:
        masterNet.connectNodes(
            NumProcessors_1, AMDws_4, "string", "NAMD_rocce_ucsd_edu_procs", blocking=True
            , splitratio=[0.22553138779262585, 0.39031385694776288])
    except:
        print "WARNING: failed to restore connection between NumProcessors_1 and AMDws_4 in network masterNet"
if Output_Directory_Browser_2 is not None and DownloadDir_12 is not None:
    try:
        masterNet.connectNodes(
            Output_Directory_Browser_2, DownloadDir_12, "directory", "path", blocking=True
            , splitratio=[0.41029417603939722, 0.32301918529073281])
    except:
        print "WARNING: failed to restore connection between Output_Directory_Browser_2 and DownloadDir_12 in network masterNet"
if AMDws_4 is not None and DownloadDir_12 is not None:
    try:
        masterNet.connectNodes(
            AMDws_4, DownloadDir_12, "GetMainURLFromList_newurl", "url", blocking=True
            , splitratio=[0.50549266266146953, 0.39728842103951723])
    except:
        print "WARNING: failed to restore connection between AMDws_4 and DownloadDir_12 in network masterNet"
masterNet.runOnNewData.value = False

if __name__=='__main__':
    from sys import argv
    lNodePortValues = []
    if (len(argv) > 1) and argv[1].startswith('-'):
        lArgIndex = 2
    else:
        lArgIndex = 1
    while lArgIndex < len(argv) and argv[lArgIndex][-3:]!='.py':
        lNodePortValues.append(argv[lArgIndex])
        lArgIndex += 1
    masterNet.setNodePortValues(lNodePortValues)
    if '--help' in argv or '-h' in argv: # show help
        masterNet.helpForNetworkAsApplication()
    elif '-w' in argv: # run without Vision and exit
         # create communicator
        from NetworkEditor.net import Communicator
        masterNet.communicator = Communicator(masterNet)
        print 'Communicator listening on port:', masterNet.communicator.port

        import socket
        f = open(argv[0]+'.sock', 'w')
        f.write("%s %i"%(socket.gethostbyname(socket.gethostname()),
                         masterNet.communicator.port))
        f.close()

        masterNet.run()

    else: # stand alone application while vision is hidden
        if '-e' in argv: # run and exit
            masterNet.run()
        elif '-r' in argv or len(masterNet.userPanels) == 0: # no user panel => run
            masterNet.run()
            mainLoopVisionToRunNetworkAsApplication(masterNet.editor)
        else: # user panel
            mainLoopVisionToRunNetworkAsApplication(masterNet.editor)





about = "The Virtual Screening workflows version 0.1 provide tools to run \n"
about = about + "AutodDck-based virtual screening. Available workflows:\n"
about = about + "  - AutoDockML - run virtual screening for sinlge receptor and multiple ligands using AutoDock\n"
about = about + "  - AutoDockVSlocal - virtual screening with AutoDock and user-provided ligand libraries \n"
about = about + "  - AutoDockVSpublic - virtual screening with AutoDock and ligand libraries on the server\n"
about = about + "  - VinaVSlocal - virtual screening using AutoDock Vina and user-provided ligand libraries\n"
about = about + "  - VinaVSpublic - virtual screening using AutoDock Vina and ligand libraries on the server\n"



#       
#           AutoDock | Raccoon2
#
#       Copyright 2013, Stefano Forli
#          Molecular Graphics Lab
#  
#     The Scripps Research Institute 
#           _  
#          (,)  T  h e
#         _/
#        (.)    S  c r i p p s
#          \_
#          (,)  R  e s e a r c h
#         ./  
#        ( )    I  n s t i t u t e
#         '
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import CADD.Raccoon2

# Raccoon stuff
import CADD.Raccoon2.HelperFunctionsN3P as hf
import RaccoonBasics as rb
import RaccoonEvents


import os, Pmw, glob
from PmwOptionMenu import OptionMenu as OptionMenuFix

import Tkinter as tk
import tkMessageBox as tmb
import tkFileDialog as tfd
from PIL import Image, ImageTk

import TkTreectrl

# mgl modules
from mglutil.events import Event, EventHandler
from mglutil.util.callback import CallbackFunction #as cb


class ReceptorTab(rb.TabBase, rb.RaccoonDefaultWidget):

    def __init__(self, app, parent, debug=False):
        rb.TabBase.__init__(self, app, parent, debug = False)
        rb.RaccoonDefaultWidget.__init__(self, parent)
        self.resource = self.app.resource
        self.dockengine = self.app.dockengine
        #self.app.eventManager.registerListener(RaccoonEvents.SetResourceEvent, self.handleResource)
        self.initIcons()        
        self.makeInterface() 


    def initIcons(self):
        """ initialize the icons for the interface"""
        icon_path = CADD.Raccoon2.ICONPATH

        f = icon_path + os.sep + 'system.png'
        self._ICON_sys = ImageTk.PhotoImage(Image.open(f))

        f = icon_path + os.sep + 'water.png'
        self._ICON_water = ImageTk.PhotoImage(Image.open(f))

        f = icon_path + os.sep + 'floppy.png'
        self._ICON_floppy = ImageTk.PhotoImage(Image.open(f))

        f = icon_path + os.sep + 'open.png'
        self._ICON_open = ImageTk.PhotoImage(Image.open(f))

        f = icon_path + os.sep + 'add.png'
        self._ICON_add = ImageTk.PhotoImage(Image.open(f))

        f = icon_path + os.sep + 'remove.png'
        self._ICON_rem = ImageTk.PhotoImage(Image.open(f))

    def makeInterface(self, resource=None):
        """ """
        self.makeInterfaceSsh()
        if resource == 'local':
            self.makeInterfaceLocal()
        elif resource == 'ssh':
            self.makeInterfaceSsh()
        elif resource == 'opal':
            self.makeInterfaceOpal()        


    def makeInterfaceLocal(self):
        self.makeInterfaceSsh()
        """ """
        pass

    def openfiles(self, event=None, files = [], quiet=False):
        """ ask for one or more rec files"""
        title = 'Select receptor file(s), flex res...' 
        filetypes = [("Supported ligand formats", ("*.pdbqt","*.PDBQT" )),
                     ("PDBQT", ("*.pdbqt", "*.PDBQT")), ("Any file type...", "*")]        
        if not files:
            files = tfd.askopenfilename(parent=self.frame, title = title, filetypes = filetypes, multiple = 1)
            files = self._listfixer(files)
        if len(files):
            self._file_checker(flist=files, quiet=quiet) 

    def opendir(self, event=None, recursive=False):
        """ scan dir(s) optionally recursively"""
        title = 'Select a directory to scan'
        if recursive:
            title += ' recursively'
        # add a GUI feedback:
        # - GUIvar 
        # - progress bar
        # progress = tk.StringVar('Processing 
        #
        dirname = tfd.askdirectory(parent=self.frame, title = title, mustexist=True)
        if dirname:
            files = hf.pathToList(dirname, pattern = '*.pdbqt', recursive = recursive) # XXX progress var here...
            #files.append(hf.pathToList(dirname, pattern = '*.PDBQT', recursive = recursive)) # XXX ?
            #hf.writeList('FOUND.log', files, addNewLine=1)
            if len(files):
                self._file_checker(files)

    def opendir_recursive(self, event=None):
        """ """
        self.opendir(recursive=True)

    def addfiles(self, accepted):
        """ """
        for rec in accepted:
            r = rec[0]
            rec_data = self.app.engine.RecBook[r]
            types = ""
            unk=''
            flex_res = '[no]'
            chains = ''
            for a in sorted(rec_data['atypes']):
                types+=" %s" % a
            types = types.strip()
            for a in sorted(rec_data['atypes_unknown']):
                unk += ' %s' % a
            unk = unk.strip()
            if rec_data['is_flexible']: # in rec_data.keys():
                #print "ISFLEXI"
                flex_res = " ".join(rec_data['flex_res'])
            chains = " ".join(rec_data['chains'])
            self.recFileManager.listbox.insert('end',rec_data['name'], chains, 
                len(rec_data['residues']), flex_res, types, unk, rec_data['filename'])
            # EVENT
            # XXX TEMPORARY
            self.app.configTab.setReceptorList()
            #elif action=='del':
                #print "FIND A WAY TO DELETE SOMETHING FROM self.tab2.recContainer"
                # find the index!
                #self.tab1.ligContainer.listbox.delete(0,END)
                # XXX Obsolete
            #    pass        
        e = RaccoonEvents.UserInputRequirementUpdate('rec')
        self.app.eventManager.dispatchEvent(e)
        self.updatereccount()

    ### 
    def deletefiles(self, nuke=False, event=None):
        if nuke == True:
            # EVENT UPDATES
            self.app.engine.removeReceptors()
            self.recFileManager.listbox.delete(0,'end')
            self.updatereccount()
        else:
            try:
                s = self.recFileManager.listbox.curselection()[0]
                sel = self.recFileManager.listbox.get(s)[0]
            except:
                #print "BB_REC> Nothing selected, leaving..."
                return
            name = sel[0]
            self.app.engine.removeReceptors([name])
            self.recFileManager.listbox.delete(s)
        e = RaccoonEvents.UserInputRequirementUpdate('rec')
        self.app.eventManager.dispatchEvent(e)
        self.updatereccount()

    def deleteallfiles(self):
        c = len(self.app.engine.RecBook.keys())
        if c == 0:
            return
        t = 'Warning'
        m = 'All %d receptors are going to be deleted\n\nAre you sure?' % c
        if not tmb.askyesno(parent=self.frame, title=t, message=m, icon='warning'):
            return
        self.deletefiles(nuke=True)


    def updatereccount(self):
        """update labels and list in the config panel in the """
        # update the count of accepted receptors
        msg = 'Accepted structures [ %s ]' % len(self.app.engine.RecBook) 
        self.recGroup.configure(tag_text = msg)
        ## XXX  THIS IS THE CONFIG PANEL ENTRY
        ## self.GUI_rec_list_container.insert('end', rec_data['name'])
        e = RaccoonEvents.ReceptorListChange()
        self.app.eventManager.dispatchEvent(e)





    def _file_checker(self, flist, quiet=False):
        """query the Raccoon engine to check if files are acceptable
            and generate a minireport...?

            then proceed to adding them to the list
        """
        table = { 'rec' : 'receptors', 'lig' : 'ligands', 'flex' : 'flex.residues',
            'error' : 'errors/duplicates', 'is_ligand': 'ligands'}

        response = self.app.engine.addReceptorList(flist)
        accepted = response['accepted']
        self.addfiles(accepted)
        rejected = response['rejected']
        #del rejected['rec']

        if not quiet:
            def close(event=None):
                win.destroy()
                
            win = Pmw.Dialog(self.frame, title='Report', 
                buttons = ('Close',), command = close)
            # XXX
            bbox = win.component('buttonbox')
            for i in range(bbox.numbuttons()):
                bbox.button(i).configure(font=self.FONT, default='disabled', **self.BORDER)

            w = tk.Frame(win.interior(), bg='white', **self.BORDER)
            tk.Label(w, text='Accepted :', justify='left',anchor='e', font=self.FONTbold, bg='white').grid(row = 1, column = 1, sticky='we')
            tk.Label(w, text=len(accepted), justify='left', font=self.FONT, bg='white', anchor='w').grid(row = 1, column = 2, sticky='we')
            tk.Frame(w,height=2,width=1,bd=1,relief='sunken').grid(row=3,column=0,sticky='we', columnspan=13)
            if len(rejected):
                tk.Label(w, text='Problematic ', justify='left',anchor='e', font=self.FONTbold, bg='white').grid(row = 4, column = 1, sticky='we')
                r = 5
                reasons = {}
                for fail in rejected: #.keys():
                    #f = fail[0]
                    e = fail[1]
                    if e in reasons.keys():
                        reasons[e] += 1
                    else:
                        reasons[e] = 1
                for err in sorted(reasons.keys()):
                    msg = "%s :" % table.get(err, 'unknown')
                    tk.Label(w, text=msg, justify='left', anchor='e',font=self.FONT, bg='white').grid(row = r, column = 1, sticky='we')
                    tk.Label(w, text=reasons[e], justify='left', anchor='w',font=self.FONT, bg='white').grid(row = r, column = 2, sticky='we')
                    r+=1
            w.pack(expand=1,fill='both', padx=10, pady=10, ipadx=5, ipady=5)
            win.activate()


    def makeInterfaceSsh(self):
        """ """
        bset = { 'bg' : '#969b9d'  } # 'width' : 22, 'height': 22, 'relief' : 'raised'}
        bset = {}
        bset.update(self.BORDER)
        self.resetFrame()
        self.recGroup = Pmw.Group(self.frame, tag_text = 'Accepted structures [ 0 ]', tag_font = self.FONTbold)

        # toolbar
        toolb = tk.Frame(self.recGroup.interior())

        if self.sysarch == 'Windows':
            bwidth = 54
        else:
            bwidth = 32
        ###### add button
        # make button
        b = tk.Button(toolb, text='Add...', compound='top', image = self._ICON_add, width=bwidth, font=self.FONTbold, **bset )
        b.pack(anchor='n', side='top')
        # make menu items
        addrec_items = [ ['Add receptors'],
                         ['Import files...', 'normal', self.openfiles],
                         [],
                         ['Scan directory...', 'normal', self.opendir],
                         ['Scan directory (recursively)...', 'normal', self.opendir_recursive],
                         # [],
                         # ['Select structure from history...', 'normal', self.opendir],
                         ]
        # make menu
        menu = rb.RacMenu(b, addrec_items, toolbar=toolb)

        ###### add button
        # make button
        b = tk.Button(toolb, text='Remove...', compound='top', image = self._ICON_rem, width=bwidth, font=self.FONTbold, **bset )
        b.pack(anchor='n', side='top', pady=1)
        # make menu items
        addrec_items = [ ['Remove receptors'],
                         ['Remove selected', 'normal', self.deletefiles],
                         ['Remove all', 'normal', self.deleteallfiles],
                         ]
        # make menu
        menu = rb.RacMenu(b, addrec_items, toolbar=toolb)
        # 
        #tk.Button(toolb, text='Settings\n&\nAlignment...', compound='top', image = self._ICON_sys, width=32, 
        #    font=self.FONTbold, **bset ).pack(anchor='n', side='top',pady=1)
        toolb.pack(side='left', anchor='w', expand=0, fill='y',pady=0)

        # files manager
        self.recFileManager = TkTreectrl.ScrolledMultiListbox(self.recGroup.interior(), bd=2)
        self.recFileManager.listbox.config(bg='white', fg='black', font=self.FONT,
                   columns = ('name', 'chains', 'res.', 'flex_res', 
                              'atom types', 'unk.types', 'filename'),
                   selectmode='extended',
                              )
        delkey = CallbackFunction(self.deletefiles, {'nuke':False})
        self.recFileManager.listbox.bind('<Delete>', delkey)

        self.recFileManager.pack(anchor='w', side='left', expand=1, fill='both')

        self.recGroup.pack(anchor='n', side='top', expand=1, fill='both')

        self.frame.pack(expand=1, fill='both',anchor='n', side='top')

    def makeInterfaceOpal(self):
        self.makeInterfaceSsh()
        """ """
        pass



#!/bin/ksh ~/.mgltools/pythonsh
########################################################################
#
#    Vision Network - Python source code - file generated by vision
#    Thursday 24 February 2011 02:28:13 
#    
#       The Scripps Research Institute (TSRI)
#       Molecular Graphics Lab
#       La Jolla, CA 92037, USA
#
# Copyright: Daniel Stoffler, Michel Sanner and TSRI
#   
# revision: Guillaume Vareille
#  
#########################################################################
#
# $Header: /opt/cvs/CADD/workflows/MDanalysis/GROMOSClustering_0.1_net.py,v 1.1 2011/04/15 16:03:51 nadya Exp $
#
# $Id: GROMOSClustering_0.1_net.py,v 1.1 2011/04/15 16:03:51 nadya Exp $
#


if __name__=='__main__':
    from sys import argv
    if '--help' in argv or '-h' in argv or '-w' in argv: # run without Vision
        withoutVision = True
        from Vision.VPE import NoGuiExec
        ed = NoGuiExec()
        from NetworkEditor.net import Network
        import os
        masterNet = Network("process-"+str(os.getpid()))
        ed.addNetwork(masterNet)
    else: # run as a stand alone application while vision is hidden
        withoutVision = False
        from Vision import launchVisionToRunNetworkAsApplication, mainLoopVisionToRunNetworkAsApplication
	if '-noSplash' in argv:
	    splash = False
	else:
	    splash = True
        masterNet = launchVisionToRunNetworkAsApplication(splash=splash)
        import os
        masterNet.filename = os.path.abspath(__file__)
from traceback import print_exc
## loading libraries ##
from AutoDockTools.VisionInterface.Adt import Adt
from Vision.StandardNodes import stdlib
from WebServices.VisionInterface.WSNodes import wslib
try:
    masterNet
except (NameError, AttributeError): # we run the network outside Vision
    from NetworkEditor.net import Network
    masterNet = Network()

masterNet.getEditor().addLibraryInstance(Adt,"AutoDockTools.VisionInterface.Adt", "Adt")

masterNet.getEditor().addLibraryInstance(stdlib,"Vision.StandardNodes", "stdlib")

masterNet.getEditor().addLibraryInstance(wslib,"WebServices.VisionInterface.WSNodes", "wslib")

from WebServices.VisionInterface.WSNodes import addOpalServerAsCategory
try:
    addOpalServerAsCategory("http://kryptonite.nbcr.net/opal2", replace=False)
except:
    pass
try:
    ## saving node GromosCluster ##
    from Adt.Macro.GromosCluster import GromosCluster
    GromosCluster_0 = GromosCluster(constrkw={}, name='GromosCluster', library=Adt)
    masterNet.addNode(GromosCluster_0,396,213)
    apply(GromosCluster_0.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
    GROMOS_ClusterFiles_kryptonite_nbcr_net_3 = GromosCluster_0.macroNetwork.nodes[2]
    GROMOS_ClusterFiles_kryptonite_nbcr_net_3.inputPortByName['expert_options'].widget.set(r"", run=False)
    GROMOS_ClusterFiles_kryptonite_nbcr_net_3.inputPortByName['prefix'].widget.set(r"", run=False)
    GROMOS_ClusterFiles_kryptonite_nbcr_net_3.inputPortByName['default_options'].widget.set(r"-o -g -dist -ev -sz -tr -ntr -clid -cl", run=False)
    GROMOS_ClusterFiles_kryptonite_nbcr_net_3.inputPortByName['localRun'].widget.set(0, run=False)
    apply(GROMOS_ClusterFiles_kryptonite_nbcr_net_3.inputPortByName['method'].widget.configure, (), {'choices': ('gromos', 'linkage', 'jarvis-patrick', 'monte-carlo', 'diagonalization')})
    GROMOS_ClusterFiles_kryptonite_nbcr_net_3.inputPortByName['method'].widget.set(r"gromos", run=False)
    GROMOS_ClusterFiles_kryptonite_nbcr_net_3.inputPortByName['execPath'].widget.set(r"", run=False)
    Entry_6 = GromosCluster_0.macroNetwork.nodes[5]
    Entry_6.inputPortByName['entry'].widget.set(r"3", run=False)

    ## saving connections for network GromosCluster ##
    GromosCluster_0.macroNetwork.freeze()
    GromosCluster_0.macroNetwork.unfreeze()

    ## modifying MacroInputNode dynamic ports
    input_Ports_1 = GromosCluster_0.macroNetwork.ipNode
    input_Ports_1.outputPorts[1].configure(name='GROMOS_ClusterFiles_kryptonite_nbcr_net_traj')
    input_Ports_1.outputPorts[2].configure(name='GROMOS_ClusterFiles_kryptonite_nbcr_net_active')
    input_Ports_1.outputPorts[3].configure(name='GROMOS_ClusterFiles_kryptonite_nbcr_net_rmsd')
    input_Ports_1.outputPorts[4].configure(name='DownloadSaveDir_path')

    ## modifying MacroOutputNode dynamic ports
    output_Ports_2 = GromosCluster_0.macroNetwork.opNode
    output_Ports_2.inputPorts[1].configure(singleConnection='auto')
    output_Ports_2.inputPorts[1].configure(name='DownloadSaveDir_output')
    GromosCluster_0.inputPorts[0].configure(name='GROMOS_ClusterFiles_kryptonite_nbcr_net_traj')
    GromosCluster_0.inputPorts[0].configure(datatype='string')
    GromosCluster_0.inputPorts[1].configure(name='GROMOS_ClusterFiles_kryptonite_nbcr_net_active')
    GromosCluster_0.inputPorts[1].configure(datatype='string')
    GromosCluster_0.inputPorts[2].configure(name='GROMOS_ClusterFiles_kryptonite_nbcr_net_rmsd')
    GromosCluster_0.inputPorts[2].configure(datatype='string')
    GromosCluster_0.inputPorts[3].configure(name='DownloadSaveDir_path')
    GromosCluster_0.inputPorts[3].configure(datatype='string')
    ## configure MacroNode input ports
    GromosCluster_0.outputPorts[0].configure(name='DownloadSaveDir_output')
    GromosCluster_0.outputPorts[0].configure(datatype='string')
    ## configure MacroNode output ports
    GromosCluster_0.shrink()
    apply(GromosCluster_0.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
except:
    print "WARNING: failed to restore GromosCluster named GromosCluster in network masterNet"
    print_exc()
    GromosCluster_0=None

try:
    ## saving node TrajectoryPDBFile ##
    from Vision.StandardNodes import FileBrowserNE
    TrajectoryPDBFile_7 = FileBrowserNE(constrkw={}, name='TrajectoryPDBFile', library=stdlib)
    masterNet.addNode(TrajectoryPDBFile_7,88,47)
    TrajectoryPDBFile_7.inputPortByName['filename'].widget.set(r"GROMOSClustering_0.1_input/trajectory.pdb", run=False)
    apply(TrajectoryPDBFile_7.configure, (), {'paramPanelImmediate': 1})
except:
    print "WARNING: failed to restore FileBrowserNE named TrajectoryPDBFile in network masterNet"
    print_exc()
    TrajectoryPDBFile_7=None

try:
    ## saving node ActiveResidueFile ##
    from Vision.StandardNodes import FileBrowserNE
    ActiveResidueFile_8 = FileBrowserNE(constrkw={}, name='ActiveResidueFile', library=stdlib)
    masterNet.addNode(ActiveResidueFile_8,412,46)
    ActiveResidueFile_8.inputPortByName['filename'].widget.set(r"GROMOSClustering_0.1_input/activeSiteResidue.pdb", run=False)
    apply(ActiveResidueFile_8.configure, (), {'paramPanelImmediate': 1})
except:
    print "WARNING: failed to restore FileBrowserNE named ActiveResidueFile in network masterNet"
    print_exc()
    ActiveResidueFile_8=None

try:
    ## saving node RMSD ##
    from Vision.StandardNodes import EntryNE
    RMSD_9 = EntryNE(constrkw={}, name='RMSD', library=stdlib)
    masterNet.addNode(RMSD_9,772,46)
    RMSD_9.inputPortByName['entry'].widget.set(r"0.08", run=False)
    apply(RMSD_9.configure, (), {'paramPanelImmediate': 1})
except:
    print "WARNING: failed to restore EntryNE named RMSD in network masterNet"
    print_exc()
    RMSD_9=None

try:
    ## saving node OutputDirectory ##
    from Vision.StandardNodes import DirBrowserNE
    OutputDirectory_10 = DirBrowserNE(constrkw={}, name='OutputDirectory', library=stdlib)
    masterNet.addNode(OutputDirectory_10,1025,38)
    OutputDirectory_10.inputPortByName['directory'].widget.set(r"GROMOSClustering_0.1_output", run=False)
    apply(OutputDirectory_10.configure, (), {'paramPanelImmediate': 1})
except:
    print "WARNING: failed to restore DirBrowserNE named OutputDirectory in network masterNet"
    print_exc()
    OutputDirectory_10=None

try:
    ## saving node DPFTemplateBrowser ##
    from Adt.Input.DPFTemplateBrowser import DPFTemplateBrowser
    DPFTemplateBrowser_11 = DPFTemplateBrowser(constrkw={}, name='DPFTemplateBrowser', library=Adt)
    masterNet.addNode(DPFTemplateBrowser_11,633,265)
    DPFTemplateBrowser_11.inputPortByName['dpf_template_file'].widget.set(r"GROMOSClustering_0.1_input/2HTY_A.dpf", run=False)
    apply(DPFTemplateBrowser_11.configure, (), {'paramPanelImmediate': 1})
except:
    print "WARNING: failed to restore DPFTemplateBrowser named DPFTemplateBrowser in network masterNet"
    print_exc()
    DPFTemplateBrowser_11=None

try:
    ## saving node GPFTemplateBrowser ##
    from Adt.Input.GPFTemplateBrowser import GPFTemplateBrowser
    GPFTemplateBrowser_12 = GPFTemplateBrowser(constrkw={}, name='GPFTemplateBrowser', library=Adt)
    masterNet.addNode(GPFTemplateBrowser_12,6,255)
    GPFTemplateBrowser_12.inputPortByName['gpf_template_file'].widget.set(r"GROMOSClustering_0.1_input/2HTY_A.gpf", run=False)
    apply(GPFTemplateBrowser_12.configure, (), {'paramPanelImmediate': 1})
except:
    print "WARNING: failed to restore GPFTemplateBrowser named GPFTemplateBrowser in network masterNet"
    print_exc()
    GPFTemplateBrowser_12=None

try:
    ## saving node MakeGpfDpfCopies ##
    from Adt.Input.MakeGpfDpfCopies import MakeGpfDpfCopies
    MakeGpfDpfCopies_13 = MakeGpfDpfCopies(constrkw={}, name='MakeGpfDpfCopies', library=Adt)
    masterNet.addNode(MakeGpfDpfCopies_13,362,466)
    apply(MakeGpfDpfCopies_13.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
except:
    print "WARNING: failed to restore MakeGpfDpfCopies named MakeGpfDpfCopies in network masterNet"
    print_exc()
    MakeGpfDpfCopies_13=None

#masterNet.run()
masterNet.freeze()

## saving connections for network GROMOSClustering-0.1 ##
if TrajectoryPDBFile_7 is not None and GromosCluster_0 is not None:
    try:
        masterNet.connectNodes(
            TrajectoryPDBFile_7, GromosCluster_0, "filename", "GROMOS_ClusterFiles_kryptonite_nbcr_net_traj", blocking=True
            , splitratio=[0.40516597457034675, 0.47883762346301628])
    except:
        print "WARNING: failed to restore connection between TrajectoryPDBFile_7 and GromosCluster_0 in network masterNet"
if ActiveResidueFile_8 is not None and GromosCluster_0 is not None:
    try:
        masterNet.connectNodes(
            ActiveResidueFile_8, GromosCluster_0, "filename", "GROMOS_ClusterFiles_kryptonite_nbcr_net_active", blocking=True
            , splitratio=[0.35790962133037152, 0.58579177190104637])
    except:
        print "WARNING: failed to restore connection between ActiveResidueFile_8 and GromosCluster_0 in network masterNet"
if RMSD_9 is not None and GromosCluster_0 is not None:
    try:
        masterNet.connectNodes(
            RMSD_9, GromosCluster_0, "string", "GROMOS_ClusterFiles_kryptonite_nbcr_net_rmsd", blocking=True
            , splitratio=[0.50918524073258298, 0.2104640996462267])
    except:
        print "WARNING: failed to restore connection between RMSD_9 and GromosCluster_0 in network masterNet"
if OutputDirectory_10 is not None and GromosCluster_0 is not None:
    try:
        masterNet.connectNodes(
            OutputDirectory_10, GromosCluster_0, "directory", "DownloadSaveDir_path", blocking=True
            , splitratio=[0.65993064333774032, 0.62912512597946058])
    except:
        print "WARNING: failed to restore connection between OutputDirectory_10 and GromosCluster_0 in network masterNet"
if GPFTemplateBrowser_12 is not None and MakeGpfDpfCopies_13 is not None:
    try:
        masterNet.connectNodes(
            GPFTemplateBrowser_12, MakeGpfDpfCopies_13, "gpf_template", "gpf_file", blocking=True
            , splitratio=[0.38882320120172753, 0.24020719445005773])
    except:
        print "WARNING: failed to restore connection between GPFTemplateBrowser_12 and MakeGpfDpfCopies_13 in network masterNet"
if DPFTemplateBrowser_11 is not None and MakeGpfDpfCopies_13 is not None:
    try:
        masterNet.connectNodes(
            DPFTemplateBrowser_11, MakeGpfDpfCopies_13, "dpf_template", "dpf_file", blocking=True
            , splitratio=[0.63661487265432926, 0.20173131060868402])
    except:
        print "WARNING: failed to restore connection between DPFTemplateBrowser_11 and MakeGpfDpfCopies_13 in network masterNet"
if GromosCluster_0 is not None and MakeGpfDpfCopies_13 is not None:
    try:
        masterNet.connectNodes(
            GromosCluster_0, MakeGpfDpfCopies_13, "DownloadSaveDir_output", "struct_dir", blocking=True
            , splitratio=[0.44295112159001504, 0.54816281093721997])
    except:
        print "WARNING: failed to restore connection between GromosCluster_0 and MakeGpfDpfCopies_13 in network masterNet"
masterNet.runOnNewData.value = False

if __name__=='__main__':
    from sys import argv
    lNodePortValues = []
    if (len(argv) > 1) and argv[1].startswith('-'):
        lArgIndex = 2
    else:
        lArgIndex = 1
    while lArgIndex < len(argv) and argv[lArgIndex][-3:]!='.py':
        lNodePortValues.append(argv[lArgIndex])
        lArgIndex += 1
    masterNet.setNodePortValues(lNodePortValues)
    if '--help' in argv or '-h' in argv: # show help
        masterNet.helpForNetworkAsApplication()
    elif '-w' in argv: # run without Vision and exit
         # create communicator
        from NetworkEditor.net import Communicator
        masterNet.communicator = Communicator(masterNet)
        print 'Communicator listening on port:', masterNet.communicator.port

        import socket
        f = open(argv[0]+'.sock', 'w')
        f.write("%s %i"%(socket.gethostbyname(socket.gethostname()),
                         masterNet.communicator.port))
        f.close()

        masterNet.run()

    else: # stand alone application while vision is hidden
        if '-e' in argv: # run and exit
            masterNet.run()
        elif '-r' in argv or len(masterNet.userPanels) == 0: # no user panel => run
            masterNet.run()
            mainLoopVisionToRunNetworkAsApplication(masterNet.editor)
        else: # user panel
            mainLoopVisionToRunNetworkAsApplication(masterNet.editor)




#       
#           AutoDock | Raccoon2
#
#       Copyright 2013, Stefano Forli
#          Molecular Graphics Lab
#  
#     The Scripps Research Institute 
#           _  
#          (,)  T  h e
#         _/
#        (.)    S  c r i p p s
#          \_
#          (,)  R  e s e a r c h
#         ./  
#        ( )    I  n s t i t u t e
#         '
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# 
# water related code
# v.0.1 start  26.1.2012
# v.0.2 rewritten class  2012.4.11
# 
#
"""
        - is RECEPTOR available?

        - is EXP_WAT  available?

        - is WAT_MAP  available?

        -for each pose:
            - clean LIG/LIG overlaps ( DeltaS = map_point )
            - clean LIG/REC overlaps ( DeltaS = map_point )
            ####### - clean FREE WATERS      ( DeltaS = map_point  ) ???
            ####### - score BRIDGING WATERS  ( DeltaH - DeltaS )
            - score BRIDGING WATERS  ( DeltaH - DeltaS )
            

    FREE WATERS issue:
        - are more favorable for a ligand, hence they should be kept?
        - are less favorab

"""

import HelperFunctionsN3P as hf
from numpy import array
#CLASH = 2.03 # min. distance between W and real atoms (lig, rec) # XXX to remove spurious water in 2zye
#CLASH = 2.15 # min. distance between W and real atoms (lig, rec)

LONELINESS_DISTANCE = 3.5 # to be used to remove lone waters not interacting with anything..b

# TODO add a function to renumber the PDBQT after removing waters?
# XXX MAKE IT A CLASS!!!


class processHydroDocking:


    def __init__(self, pose=None,           # advanced pose to process [text, coords, water_bridge, ...]
                  gridmap=None,              # grid map (file/dictionary)

                  # distance parms
                  mapdistrange=1.0,     # distance around W coords to look for grid points
                  clust_tol = 2.0,      # cluster tolerance
                  interact_dist = 3.5,  # W-RecAtom interaction distance
                  clash= 2.03,          # min.distance allowed between W/atoms (lig, rec) # XX to remove spurious watre
                  loneliness_dist = 3.5, # max distance between W and interacting rec atoms

                  # energy parms
                  cutoff_weak = -0.35,  # weak water e.cutoff
                  cutoff_strong = -0.5, # strong water e.cutoff
                  desolvEntropy=-0.2,   # desolvation entropy
                  conservedwaterentropy=0,  # add an entropy penalty also for conserved waters

                  ignore_types = ['H', 'HD', 'W'], # atoms ignored for lig_overlap
                  debug = False):   # ignore atom types for contacts

        if gridmap == None:
            #raise "No water map provided"
            print "No water map provided"
            return False
        print "WARNING processHydroDocking:_init_> clash distance [%2.3f] is extremely low! Test with some case studies" % clash
        self.gridmap = gridmap
        self.pose = pose
        self.mapdistrange = mapdistrange
        self.clust_tol = clust_tol
        self.interact_dist = interact_dist
        self.clash = clash
        self.loneliness_dist = loneliness_dist

        self.cutoff_weak = cutoff_weak
        self.cutoff_strong = cutoff_strong
        self.desolvEntropy = desolvEntropy
        self.ignore_types = ignore_types
        self.conservedwaterentropy = conservedwaterentropy

        self.qs = hf.QuickStop()

        self.debug = debug

        if not (pose == None) and not (gridmap == None):
            self.process

    
    def process(self):

        # w-lig overlap -> ['water_lig_over']
        self.ligOverlapWaters() 
        # w-rec overlap -> ['water_rec_over']
        self.recOverlapWaters()

        # get energies for ['water_bridge_score'], ['water_lig_over_penalty'], ['water_rec_over_penalty']
        self.getWaterGridEnergy()

        # cluster ['water_bridge']
        self.clusterWaters()

        # clean-up extra_weak waters (increment ['water_rec_over_penalty'], if necessary
        self.weakWaterCleanup()

        # score waters and update PDBQT
        self.rankWaters()

        # normalize ligand energy poses['e'] ; poses['leff']
        self.normalizeWaterScore()

        return self.pose






    def getMapPoints(self, coords=None, distance=1.0): # mapdata, coords=None, distance=1.0):
        """ get points around (distance) of coordinates (coords)
        """
        # XXX TODO it should become interpolator!
        if coords == None:
            print "getMapPoints> ERROR: no coordinates!"
            return False
        data = self.gridmap['values']
        v_min = self.gridmap['min']
        v_max = self.gridmap['max']
        spacing = self.gridmap['spacing']
        pts = self.gridmap['pts']
        harvesting = []

        pt_scan = int( round(float(distance)/(spacing)) )
        if pt_scan == 0: pt_scan = 1 # at least one point around
        if self.debug: print "Grid range: %2.2f [ +/- %d points : %2.2f ]" % (distance,pt_scan, pt_scan*spacing), 
        best = 0
        warning_issued = False
        if (coords[0] < v_max[0]) and (coords[0] > v_min[0]):
            if (coords[1] < v_max[1]) and (coords[1] > v_min[1]):
                if (coords[2] < v_max[2]) and (coords[2] > v_min[2]):
                    z_pt = int(round((coords[2] - v_min[2])/spacing))
                    y_pt = int(round((coords[1] - v_min[1])/spacing))
                    x_pt = int(round((coords[0] - v_min[0])/spacing))
                    for x_ofs in range(-pt_scan, pt_scan+1): 
                        x = x_pt + x_ofs
                        if x < 0 :
                            harvesting.append(0)
                            break
                        for y_ofs in range(-pt_scan, pt_scan+1): 
                            y = y_pt + y_ofs
                            if y < 0 :
                                harvesting.append(0)
                                break
                            for z_ofs in range(-pt_scan, pt_scan+1): 
                                z = z_pt + z_ofs
                                if z < 0 :
                                    harvesting.append(0)
                                    break
                                try:
                                    harvesting.append( data[z,y,x] )
                                    if data[z,y,x] < best:
                                        best = data[z,y,x]
                                except:
                                    if not warning_issued:
                                        msg = ('*** WaterProcessing> Warning! '
                                               'Wat(%2.3f,%2.3f,%2.3f) '
                                               'close to gridbox edge ***' % (coords[0], coords[1], coords[2])
                                               )
                                        best = 0
                                        print msg
                                        warning_issued = True
                                    
                                    harvesting.append( 0 )
                if self.debug: print "genMapPoints> %d pts [ best: %2.2f ]" % (len(harvesting), best),
                return harvesting, best
        return False



    def ligOverlapWaters(self): 
        """
        remove ligand-overlapping w-atoms in a pose
        """
        #, cutoff = CLASH, 
        ignore_types = self.ignore_types
        #ignore_types = [ 'W' ] 
        # add a larger tolerance for HD?

        #cutoff = 2.4

        cutoff = self.clash ** 2
        log = []
        overlapping = []
        for w in self.pose['water_bridge']:
            try:
                for a in self.pose['text']:
                    if not w == a:
                        if isAtom(a) and  (not getAtype(a) in self.ignore_types):
                            d = dist(w, a, sq=0)
                            #if d < self.clash:
                            if d < cutoff:
                                overlapping.append(w)
                                self.pose['text'].remove(w)
                                raise self.qs
            except:
                pass
        if self.debug:
            print "Processed: %d | Lig-overlap %d" % (len(self.pose['water_bridge']), len(overlapping) )
        for w in overlapping:
            idx = self.pose['water_bridge'].index(w)
            self.pose['water_bridge'].remove(w)
            self.pose['water_bridge_contacts'].pop(idx)

        self.pose['water_over_lig'] = overlapping
                    
    def recOverlapWaters(self): # pose, cutoff = CLASH,
        #ignore_types = ['HD', 'H', 'W'] ):
        """ INPUT : pdb lig (multi)model and pdb target model
            OUTPUT: cleaned pdb lig model
         """
        #print "WARNING! missing interactions! Line B1"
        overlapping = []
        atom_pool = []
        cutoff = self.clash**2
        #cutoff += 0.5
        #cutoff = 0
        #for a in self.pose['vdw_contacts']: # PDBQT, no-HD   # XXX TODO BUG Line B1
        for aset in self.pose['water_bridge_contacts']: # PDBQT, no-HD   # XXX TODO BUG Line B1
            for a in aset:
                if not hf.getAtype(a) in self.ignore_types:
                    atom_pool.append(a)
        if self.debug:
            fp = open('DEBUG_rec_overlap.pdb','w')
            fp.write("REMARK  WATERS REMOVED BY RECEPTOR CLASHES")
        for w in self.pose['water_bridge']:
            try: # try/except is faster
                for a in atom_pool:
                    #if hf.dist(w, a, sq=False) < self.clash:
                    if hf.dist(w, a, sq=False) < cutoff:
                        overlapping.append(w)
                        self.pose['text'].remove(w)
                        #print "\n\n\n ####  REMOVED", w
                        if self.debug:
                            fp.write(w+"\n")
                            fp.write(a+"\n")
                        raise self.qs
            except:
                pass 
        if self.debug:
            fp.close()
            print "Processed: %d | Rec-overlap %d" % (len(self.pose['water_bridge']), len(overlapping) )
        for w in overlapping:
            idx = self.pose['water_bridge'].index(w)
            del self.pose['water_bridge_contacts'][idx]
            self.pose['water_bridge'].remove(w)
        self.pose['water_over_rec'] = overlapping

    def getWaterGridEnergy(self): 

        #                    cutoff_weak = -0.35,   # weak water e.cutoff
        #                    cutoff_strong = -0.5,  # strong water e.cutoff
        #                    desolvEntropy=-0.2,    # desolvation entropy
        #                    mapdistrange = 0.5):   

        # TODO  to be used to 'minimize' the water position too?
        #       looking for the best spot for a water
        #       and increase the energy accordingly?
        # calculate the score for bridging waters;
        self.pose['water_bridge_scores'] = []
        for w in self.pose['water_bridge']:
            points, best_point = self.getMapPoints( coords = hf.atomCoord(w),
                                         distance = self.mapdistrange)
            self.pose['water_bridge_scores'].append(best_point)
        # calculate ligand-overlapping penalties
        self.pose['water_over_lig_penalty'] = []
        for w in self.pose['water_over_lig']:
            points, best_point = self.getMapPoints( coords = atomCoord(w),
                                        distance = self.mapdistrange)
            self.pose['water_over_lig_penalty'].append( -best_point)
        # calculate rec-overlapping penalties
        self.pose['water_over_rec_penalty'] = []
        for w in self.pose['water_over_rec']:
            self.pose['water_over_rec_penalty'].append(-self.desolvEntropy)


    def clusterWaters(self): #pose, clust_tolerance=2.0, desolvEntropy=0):
        # cluster waters
        # XXX
        # energy from the cluster?
        # once 3 waters are reclustered (3->1)
        # two desolvEntropy penalties should be issued?
        # XXX
        unclustered_waters = self.pose['water_bridge'][:] # XXX NOT SURE IT"S NECESSARY

        # cluster waters
        water_clusters = hf.clusterAtoms(unclustered_waters, tol=self.clust_tol)

        # calculate clustering energy
        self.pose['water_cluster_penalty'] = []
        remove_from_text = []
        for pop in water_clusters:
            if len(pop)>1:
                this_cluster_centroid = hf.avgCoord(pop)
                this_cluster_penalty = -self.desolvEntropy * (len(pop)-1)
                this_cluster_energy = 0
                this_cluster_contacts = []
                closest_to_centroid = None
                closest_dist = 999999
                for w in pop:
                    # get the w_index + w_score
                    idx = self.pose['water_bridge'].index(w)
                    this_cluster_energy += self.pose['water_bridge_scores'][idx]
                    remove_from_text.append(w)
                    # find water closest to centroid (to be conserved)
                    # get the distance from the cluster average
                    centr_dist =  hf.quickdist( hf.atomCoord(w), this_cluster_centroid, sq = False)
                    a = [ hf.makePdb(coord=this_cluster_centroid) ]
                    #writeList('centroid.pdb', a)
                    if centr_dist <= closest_dist:
                        closest_dist = centr_dist
                        closest_to_centroid = w
                    del self.pose['water_bridge'][idx]
                    del self.pose['water_bridge_scores'][idx]
                    # update contacts
                    for a in self.pose['water_bridge_contacts'][idx]:
                        if not a in this_cluster_contacts:
                            this_cluster_contacts.append(a)
                    del self.pose['water_bridge_contacts'][idx]
                # update PDBQT text
                # XXX DEBUGGINGH
                #print type(remove_from_text), remove_from_text[0]
                #writeList('REMOVING_WATERS.pdb', remove_from_text)
                #writeList('current_ligand.pdbqt', pose['text'])

                for w in remove_from_text:
                    if not w == closest_to_centroid:
                        try:
                            text_idx = self.pose['text'].index(w)

                            del self.pose['text'][text_idx]
                        except:
                            pass
                # use the closest to be updated with clustering coords
                closest_index = self.pose['text'].index(closest_to_centroid)
                water = self.pose['text'][closest_index]
                coord_text = "%8.3f%8.3f%8.3f" % (this_cluster_centroid[0], 
                                                  this_cluster_centroid[1],
                                                  this_cluster_centroid[2])
                cluster_water = water[0:30]+coord_text+water[54:] # XXX ugly, but it works...
                self.pose['text'][closest_index] = cluster_water
                # update water_bridge list+score, water_cluster_penalty list
                self.pose['water_bridge'].append(cluster_water)
                self.pose['water_bridge_scores'].append(this_cluster_energy)
                self.pose['water_cluster_penalty'].append(this_cluster_penalty)
                self.pose['water_bridge_contacts'].append(this_cluster_contacts)
                if self.debug:
                    print "CLUSTER SIZE", len(pop)
                    print "CLUSTER ENRG", this_cluster_energy
                    print "CLUSTER PENL", this_cluster_penalty
                    #print "CLUSTER FINL", clust_energy+clust_penalty
                    print "=============="

    def normalizeWaterScore(self): #pose,entropyPenalty=False):

        # keep track of the original energy values
        if self.debug:
            old_e =  self.pose['energy']
            old_leff = self.pose['leff']

        # rec_displaced waters
        rec_displaced = 0
        for p in self.pose['water_over_rec_penalty']:
            rec_displaced += p
        #    print p # XXX
        #print "RECDISP", rec_displaced  # XXX 
        
        # lig_displaced waters
        lig_displaced = 0
        for p in self.pose['water_over_lig_penalty']:
            lig_displaced += p

        # clustering-displaced
        clust_displaced = 0
        for p in self.pose['water_cluster_penalty']:
            clust_displaced += p

        # count heavy at for lig.efficiency
        heavy_atoms = 0
        for a in self.pose['text']:
            if hf.isAtom(a) and ( not hf.getAtype(a) in ['HD', 'W'] ):
                heavy_atoms += 1

        # energy correction
        if self.conservedwaterentropy:
            # XXX read the penalty from a variable!
            entropy = (0.2)*len(self.pose['water_bridge'])
            #print " ENTROPY PENALTY = ", entropy
        else:
            entropy = 0
        self.pose['energy'] += lig_displaced + rec_displaced + clust_displaced + entropy

        # lig.efficiency correction
        self.pose['leff'] = self.pose['energy'] / float( heavy_atoms )


        if self.debug:
            #print "\n# PENALTY:\t +%2.3f [rec: %2.3f, lig: %2.3f, clust: %2.3f ]" % (rec_displaced+lig_displaced+clust_displaced,
            #        rec_displaced, lig_displaced, clust_displaced)
            print "  Penalties:"
            print "   - lig overlap     : %2.3f" % lig_displaced
            print "   - rec overlap     : %2.3f" % rec_displaced
            print "   - cluster penalty : %2.3f" % clust_displaced
            print "   - extra entropy   : %2.3f" % entropy

            print "   ENERGY:\t %2.3f [ old: %2.3f ]" %  (self.pose['energy'], old_e)
            print "   L.EFF.:\t %2.3f [ old: %2.3f ]\n" %  (self.pose['leff'], old_leff)
        #return pose



    def rankWaters(self): #pose, cutoff_weak, cutoff_strong):
        # keep track of the original energy values
        if self.debug:
            old_e =  pose['energy']
            old_leff = pose['leff']

        rank_line = "REMARK  %s water ( score: %2.2f )\n"
        self.pose['water_bridge_rank'] = []
        for idx in range(len(self.pose['water_bridge'])):
            water = self.pose['water_bridge'][idx]
            score = self.pose['water_bridge_scores'][idx]

            if score < self.cutoff_strong:
                # strongly bound
                rank = 'STRONG'
                score_comment = rank_line % (rank, score)
            elif score < self.cutoff_weak:
                # weakly bound
                rank = 'WEAK'
                score_comment = rank_line % (rank, score)
            else:
                # transient trapped/bulk water
                rank = 'TRAPPED/BULK'
                score_comment = rank_line % (rank, score)
            self.pose['water_bridge_rank'].append( [ rank, score ])
            # update the PDBQT text
            mark = self.pose['text'].index(water)
            txt = self.pose['text'][mark]
            self.pose['text'][mark] = score_comment + txt
        if self.debug:
            for w in self.pose['water_bridge_rank']:
                print "W:", w

    def weakWaterCleanup(self): #
        #pose, cutoff_weak = -0.35, cutoff_strong=-0.5, interact_dist=3.5 ):
        """ clean-up extra_weak waters (increment ['water_rec_over_penalty'], if necessary"""
        org_waters = len(self.pose['water_bridge'])
        #ignore_types = ['HD']

        lone_water = []
        interact_dist = self.interact_dist**2
        hb_types = ['OA', 'NA', 'SA', 'N', 'O', 'OA', 'NA', 'SA']
        hb_types += hf.METALS 
        for idx in range(len(self.pose['water_bridge'])):
            water = self.pose['water_bridge'][idx]
            score = self.pose['water_bridge_scores'][idx]
            # analyze only waters that are not ranked strong
            if score >= self.cutoff_strong: 
                atom_pool = [ a for a in self.pose['water_bridge_contacts'][idx] if hf.getAtype(a) in hb_types] 
                if self.debug:
                    hf.writeList('ATOM_POOL_%d_HB_%d.pdb' % (idx, sess), atom_pool, addNewLine=True)
                lone_water.append(water)
                try:
                    for a in atom_pool:
                        if (dist(water, a, sq=0)<= interact_dist):
                            #print "FOUND MATE",
                            #print dist(water, a, sq=True)
                            #print a.strip()
                            #print "------------------"
                            raise self.qs
                    #print "NO MATE FOR THIS", score
                    #print water.strip()
                except:
                    lone_water.remove(water)
                            
        for w in lone_water:
            idx = self.pose['water_bridge'].index(w)
            del self.pose['water_bridge'][idx]
            del self.pose['water_bridge_contacts'][idx]
        
            # EXPERIMENTAL
            #w_score = self.pose['water_bridge_scores'][idx]
            #if w_score > 
            # XXX THIS shouldn't happen:
            # a weakly bound water could be hanging in the bulk, but it should be accounted for
            # ...? 
            # how to describe the bulk solvent?
            # combine desolv map + discrete waters?
            # 

            self.pose['water_over_rec_penalty'].append( -self.pose['water_bridge_scores'].pop(idx) )
            # update PDBQT text
            #print pose['water_over_rec_penalty'] # XXX
            text_idx = self.pose['text'].index(w)
            del self.pose['text'][text_idx]


        if self.debug:
            print "LONE WATERS # REMOVED: ", (org_waters- len(self.pose['water_bridge']) )


            
        




#!/bin/ksh ~/.mgltools/pythonsh
########################################################################
#
#    Vision Network - Python source code - file generated by vision
#    Wednesday 23 February 2011 18:15:15 
#    
#       The Scripps Research Institute (TSRI)
#       Molecular Graphics Lab
#       La Jolla, CA 92037, USA
#
# Copyright: Daniel Stoffler, Michel Sanner and TSRI
#   
# revision: Guillaume Vareille
#  
#########################################################################
#
# $Header: /opt/cvs/CADD/workflows/docking/AutoGrid_0.1_net.py,v 1.1 2011/04/15 16:16:48 nadya Exp $
#
# $Id: AutoGrid_0.1_net.py,v 1.1 2011/04/15 16:16:48 nadya Exp $
#


if __name__=='__main__':
    from sys import argv
    if '--help' in argv or '-h' in argv or '-w' in argv: # run without Vision
        withoutVision = True
        from Vision.VPE import NoGuiExec
        ed = NoGuiExec()
        from NetworkEditor.net import Network
        import os
        masterNet = Network("process-"+str(os.getpid()))
        ed.addNetwork(masterNet)
    else: # run as a stand alone application while vision is hidden
        withoutVision = False
        from Vision import launchVisionToRunNetworkAsApplication, mainLoopVisionToRunNetworkAsApplication
	if '-noSplash' in argv:
	    splash = False
	else:
	    splash = True
        masterNet = launchVisionToRunNetworkAsApplication(splash=splash)
        import os
        masterNet.filename = os.path.abspath(__file__)
from traceback import print_exc
## loading libraries ##
from AutoDockTools.VisionInterface.Adt import Adt
from WebServices.VisionInterface.WSNodes import wslib
from Vision.StandardNodes import stdlib
try:
    masterNet
except (NameError, AttributeError): # we run the network outside Vision
    from NetworkEditor.net import Network
    masterNet = Network()

masterNet.getEditor().addLibraryInstance(Adt,"AutoDockTools.VisionInterface.Adt", "Adt")

masterNet.getEditor().addLibraryInstance(wslib,"WebServices.VisionInterface.WSNodes", "wslib")

masterNet.getEditor().addLibraryInstance(stdlib,"Vision.StandardNodes", "stdlib")

from WebServices.VisionInterface.WSNodes import addOpalServerAsCategory
try:
    addOpalServerAsCategory("http://ws.nbcr.net/opal2", replace=False)
except:
    pass
try:
    addOpalServerAsCategory("http://kryptonite.nbcr.net/opal2", replace=False)
except:
    pass
try:
    ## saving node ComputeGrids ##
    from Adt.Macro.ComputeGrids import ComputeGrids
    ComputeGrids_0 = ComputeGrids(constrkw={}, name='ComputeGrids', library=Adt)
    masterNet.addNode(ComputeGrids_0,391,290)
    apply(ComputeGrids_0.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
    prepareGPF_kryptonite_nbcr_net_4 = ComputeGrids_0.macroNetwork.nodes[3]
    prepareGPF_kryptonite_nbcr_net_4.inputPortByName['singlelib'].widget.set(r"", run=False)
    prepareGPF_kryptonite_nbcr_net_4.inputPortByName['r_url'].widget.set(r"", run=False)
    prepareGPF_kryptonite_nbcr_net_4.inputPortByName['zpoints'].widget.set(r"", run=False)
    prepareGPF_kryptonite_nbcr_net_4.inputPortByName['filter_file_url'].widget.set(r"", run=False)
    apply(prepareGPF_kryptonite_nbcr_net_4.inputPortByName['lib'].widget.configure, (), {'choices': ('sample', 'NCIDS_SC', 'NCI_DS1', 'NCI_DS2', 'oldNCI', 'human_metabolome', 'chembridge_building_blocks', 'drugbank_nutraceutics', 'drugbank_smallmol', 'fda_approved')})
    prepareGPF_kryptonite_nbcr_net_4.inputPortByName['lib'].widget.set(r"", run=False)
    prepareGPF_kryptonite_nbcr_net_4.inputPortByName['ypoints'].widget.set(r"", run=False)
    prepareGPF_kryptonite_nbcr_net_4.inputPortByName['xcenter'].widget.set(r"auto", run=False)
    prepareGPF_kryptonite_nbcr_net_4.inputPortByName['p'].widget.set(r"", run=False)
    prepareGPF_kryptonite_nbcr_net_4.inputPortByName['o'].widget.set(r"", run=False)
    prepareGPF_kryptonite_nbcr_net_4.inputPortByName['zcenter'].widget.set(r"auto", run=False)
    prepareGPF_kryptonite_nbcr_net_4.inputPortByName['v'].widget.set(0, run=False)
    prepareGPF_kryptonite_nbcr_net_4.inputPortByName['userlib'].widget.set(r"", run=False)
    prepareGPF_kryptonite_nbcr_net_4.inputPortByName['xpoints'].widget.set(r"", run=False)
    prepareGPF_kryptonite_nbcr_net_4.inputPortByName['localRun'].widget.set(0, run=False)
    prepareGPF_kryptonite_nbcr_net_4.inputPortByName['ycenter'].widget.set(r"auto", run=False)
    prepareGPF_kryptonite_nbcr_net_4.inputPortByName['execPath'].widget.set(r"", run=False)
    autogrid_kryptonite_nbcr_net_5 = ComputeGrids_0.macroNetwork.nodes[4]
    autogrid_kryptonite_nbcr_net_5.inputPortByName['infile_url'].widget.set(r"", run=False)
    autogrid_kryptonite_nbcr_net_5.inputPortByName['l'].widget.set(r"output.glg", run=False)
    autogrid_kryptonite_nbcr_net_5.inputPortByName['o'].widget.set(0, run=False)
    autogrid_kryptonite_nbcr_net_5.inputPortByName['p'].widget.set(r"", run=False)
    autogrid_kryptonite_nbcr_net_5.inputPortByName['localRun'].widget.set(0, run=False)
    autogrid_kryptonite_nbcr_net_5.inputPortByName['execPath'].widget.set(r"", run=False)
    GetURLFromList_8 = ComputeGrids_0.macroNetwork.nodes[7]
    GetURLFromList_8.inputPortByName['ext'].widget.set(r"gpf", run=False)

    ## saving connections for network ComputeGrids ##
    ComputeGrids_0.macroNetwork.freeze()
    ComputeGrids_0.macroNetwork.unfreeze()

    ## modifying MacroInputNode dynamic ports
    input_Ports_1 = ComputeGrids_0.macroNetwork.ipNode
    input_Ports_1.outputPorts[1].configure(name='GetComputeGridsInputs_ligands')
    input_Ports_1.outputPorts[2].configure(name='GetComputeGridsInputs_receptor_pdbqt')
    input_Ports_1.outputPorts[3].configure(name='GetComputeGridsInputs_gpf_obj')

    ## modifying MacroOutputNode dynamic ports
    output_Ports_2 = ComputeGrids_0.macroNetwork.opNode
    output_Ports_2.inputPorts[1].configure(singleConnection='auto')
    output_Ports_2.inputPorts[2].configure(singleConnection='auto')
    output_Ports_2.inputPorts[1].configure(name='MakeAutogridResultObj_autogrid_result_obj')
    output_Ports_2.inputPorts[2].configure(name='GetMainURLFromList_newurl')
    ComputeGrids_0.inputPorts[0].configure(name='GetComputeGridsInputs_ligands')
    ComputeGrids_0.inputPorts[0].configure(datatype='LigandDB')
    ComputeGrids_0.inputPorts[1].configure(name='GetComputeGridsInputs_receptor_pdbqt')
    ComputeGrids_0.inputPorts[1].configure(datatype='receptor_prepared')
    ComputeGrids_0.inputPorts[2].configure(name='GetComputeGridsInputs_gpf_obj')
    ComputeGrids_0.inputPorts[2].configure(datatype='gpf_template')
    ## configure MacroNode input ports
    ComputeGrids_0.outputPorts[0].configure(name='MakeAutogridResultObj_autogrid_result_obj')
    ComputeGrids_0.outputPorts[0].configure(datatype='autogrid_results')
    ComputeGrids_0.outputPorts[1].configure(name='GetMainURLFromList_newurl')
    ComputeGrids_0.outputPorts[1].configure(datatype='string')
    ## configure MacroNode output ports
    ComputeGrids_0.shrink()
    apply(ComputeGrids_0.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
except:
    print "WARNING: failed to restore ComputeGrids named ComputeGrids in network masterNet"
    print_exc()
    ComputeGrids_0=None

try:
    ## saving node WebBrowser ##
    from WebServices.VisionInterface.WSNodes import WebBrowserNode
    WebBrowser_9 = WebBrowserNode(constrkw={}, name='WebBrowser', library=wslib)
    masterNet.addNode(WebBrowser_9,310,412)
    apply(WebBrowser_9.inputPortByName['url'].configure, (), {'defaultValue': None})
    WebBrowser_9.inputPortByName['url'].rebindWidget()
    WebBrowser_9.inputPortByName['url'].widget.set(r"", run=False)
    WebBrowser_9.inputPortByName['url'].unbindWidget()
    apply(WebBrowser_9.configure, (), {'paramPanelImmediate': 1})
except:
    print "WARNING: failed to restore WebBrowserNode named WebBrowser in network masterNet"
    print_exc()
    WebBrowser_9=None

try:
    ## saving node PublicServerLigandDB ##
    from Adt.Input.PublicServerLigandDB import PublicServerLigandDB
    PublicServerLigandDB_10 = PublicServerLigandDB(constrkw={}, name='PublicServerLigandDB', library=Adt)
    masterNet.addNode(PublicServerLigandDB_10,16,40)
    PublicServerLigandDB_10.inputPortByName['server_lib'].widget.set(r"sample", run=False)
except:
    print "WARNING: failed to restore PublicServerLigandDB named PublicServerLigandDB in network masterNet"
    print_exc()
    PublicServerLigandDB_10=None

try:
    ## saving node PrepareReceptor ##
    from Adt.Macro.PrepareReceptor import PrepareReceptor
    PrepareReceptor_11 = PrepareReceptor(constrkw={}, name='PrepareReceptor', library=Adt)
    masterNet.addNode(PrepareReceptor_11,408,153)
    apply(PrepareReceptor_11.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
    Pdb2pqrWS_14 = PrepareReceptor_11.macroNetwork.nodes[2]
    Pdb2pqrOpalService_ws_nbcr_net_18 = Pdb2pqrWS_14.macroNetwork.nodes[3]
    Pdb2pqrOpalService_ws_nbcr_net_18.inputPortByName['noopt'].widget.set(0, run=False)
    Pdb2pqrOpalService_ws_nbcr_net_18.inputPortByName['phi'].widget.set(0, run=False)
    Pdb2pqrOpalService_ws_nbcr_net_18.inputPortByName['psi'].widget.set(0, run=False)
    Pdb2pqrOpalService_ws_nbcr_net_18.inputPortByName['verbose'].widget.set(1, run=False)
    Pdb2pqrOpalService_ws_nbcr_net_18.inputPortByName['chain'].widget.set(0, run=False)
    Pdb2pqrOpalService_ws_nbcr_net_18.inputPortByName['nodebump'].widget.set(0, run=False)
    Pdb2pqrOpalService_ws_nbcr_net_18.inputPortByName['chi'].widget.set(0, run=False)
    Pdb2pqrOpalService_ws_nbcr_net_18.inputPortByName['ligand'].widget.set(r"", run=False)
    Pdb2pqrOpalService_ws_nbcr_net_18.inputPortByName['hbond'].widget.set(0, run=False)
    Pdb2pqrOpalService_ws_nbcr_net_18.inputPortByName['with_ph'].widget.set(r"", run=False)
    apply(Pdb2pqrOpalService_ws_nbcr_net_18.inputPortByName['forcefield'].widget.configure, (), {'choices': ('AMBER', 'CHARMM', 'PARSE', 'TYL06')})
    Pdb2pqrOpalService_ws_nbcr_net_18.inputPortByName['forcefield'].widget.set(r"AMBER", run=False)
    Pdb2pqrOpalService_ws_nbcr_net_18.inputPortByName['clean'].widget.set(0, run=False)
    Pdb2pqrOpalService_ws_nbcr_net_18.inputPortByName['inId'].widget.set(r"", run=False)
    Pdb2pqrOpalService_ws_nbcr_net_18.inputPortByName['apbs_input'].widget.set(0, run=False)
    apply(Pdb2pqrOpalService_ws_nbcr_net_18.inputPortByName['ffout'].widget.configure, (), {'choices': ('AMBER', 'CHARMM', 'PARSE', 'TYL06')})
    Pdb2pqrOpalService_ws_nbcr_net_18.inputPortByName['ffout'].widget.set(r"", run=False)
    Pdb2pqrOpalService_ws_nbcr_net_18.inputPortByName['localRun'].widget.set(0, run=False)
    Pdb2pqrOpalService_ws_nbcr_net_18.inputPortByName['rama'].widget.set(0, run=False)
    Pdb2pqrOpalService_ws_nbcr_net_18.inputPortByName['execPath'].widget.set(r"", run=False)
    Pdb2pqrOpalService_ws_nbcr_net_18.inputPortByName['assign_only'].widget.set(0, run=False)
    GetURLFromList_19 = Pdb2pqrWS_14.macroNetwork.nodes[4]
    GetURLFromList_19.inputPortByName['ext'].widget.set(r"pqr", run=False)

    ## saving connections for network Pdb2pqrWS ##
    Pdb2pqrWS_14.macroNetwork.freeze()
    Pdb2pqrWS_14.macroNetwork.unfreeze()

    ## modifying MacroInputNode dynamic ports
    input_Ports_15 = Pdb2pqrWS_14.macroNetwork.ipNode
    input_Ports_15.outputPorts[1].configure(name='CheckFileFormat_value')

    ## modifying MacroOutputNode dynamic ports
    output_Ports_16 = Pdb2pqrWS_14.macroNetwork.opNode
    output_Ports_16.inputPorts[1].configure(singleConnection='auto')
    output_Ports_16.inputPorts[2].configure(singleConnection='auto')
    output_Ports_16.inputPorts[1].configure(name='UpdateReceptor_receptor_obj')
    output_Ports_16.inputPorts[2].configure(name='UpdateReceptor_pdb2pqr_result')
    Pdb2pqrWS_14.inputPorts[0].configure(name='CheckFileFormat_value')
    Pdb2pqrWS_14.inputPorts[0].configure(datatype='receptor')
    ## configure MacroNode input ports
    Pdb2pqrWS_14.outputPorts[0].configure(name='UpdateReceptor_receptor_obj')
    Pdb2pqrWS_14.outputPorts[0].configure(datatype='receptor')
    Pdb2pqrWS_14.outputPorts[1].configure(name='UpdateReceptor_pdb2pqr_result')
    Pdb2pqrWS_14.outputPorts[1].configure(datatype='string')
    ## configure MacroNode output ports
    Pdb2pqrWS_14.shrink()
    PrepareReceptorWS_21 = PrepareReceptor_11.macroNetwork.nodes[3]
    PrepareReceptorOpalService_ws_nbcr_net_25 = PrepareReceptorWS_21.macroNetwork.nodes[3]
    PrepareReceptorOpalService_ws_nbcr_net_25.inputPortByName['o'].widget.set(r"", run=False)
    PrepareReceptorOpalService_ws_nbcr_net_25.inputPortByName['v'].widget.set(0, run=False)
    PrepareReceptorOpalService_ws_nbcr_net_25.inputPortByName['localRun'].widget.set(0, run=False)
    PrepareReceptorOpalService_ws_nbcr_net_25.inputPortByName['execPath'].widget.set(r"", run=False)
    GetURLFromList_26 = PrepareReceptorWS_21.macroNetwork.nodes[4]
    GetURLFromList_26.inputPortByName['ext'].widget.set(r"pdbqt", run=False)
    DownloadToFile_27 = PrepareReceptorWS_21.macroNetwork.nodes[5]
    DownloadToFile_27.inputPortByName['overwrite'].widget.set(1, run=False)

    ## saving connections for network PrepareReceptorWS ##
    PrepareReceptorWS_21.macroNetwork.freeze()
    PrepareReceptorWS_21.macroNetwork.unfreeze()

    ## modifying MacroInputNode dynamic ports
    input_Ports_22 = PrepareReceptorWS_21.macroNetwork.ipNode
    input_Ports_22.outputPorts[1].configure(name='CheckFileFormat_value')
    input_Ports_22.outputPorts[2].configure(name='PrepareReceptorOpalService_ws_nbcr_net_C')

    ## modifying MacroOutputNode dynamic ports
    output_Ports_23 = PrepareReceptorWS_21.macroNetwork.opNode
    output_Ports_23.inputPorts[1].configure(singleConnection='auto')
    output_Ports_23.inputPorts[2].configure(singleConnection='auto')
    output_Ports_23.inputPorts[1].configure(name='UpdateReceptor_receptor_prepared_obj')
    output_Ports_23.inputPorts[2].configure(name='UpdateReceptor_receptor_result')
    PrepareReceptorWS_21.inputPorts[0].configure(name='CheckFileFormat_value')
    PrepareReceptorWS_21.inputPorts[0].configure(datatype='receptor')
    PrepareReceptorWS_21.inputPorts[1].configure(name='PrepareReceptorOpalService_ws_nbcr_net_C')
    PrepareReceptorWS_21.inputPorts[1].configure(datatype='boolean')
    ## configure MacroNode input ports
    PrepareReceptorWS_21.outputPorts[0].configure(name='UpdateReceptor_receptor_prepared_obj')
    PrepareReceptorWS_21.outputPorts[0].configure(datatype='receptor_prepared')
    PrepareReceptorWS_21.outputPorts[1].configure(name='UpdateReceptor_receptor_result')
    PrepareReceptorWS_21.outputPorts[1].configure(datatype='string')
    ## configure MacroNode output ports
    PrepareReceptorWS_21.shrink()

    ## saving connections for network PrepareReceptor ##
    PrepareReceptor_11.macroNetwork.freeze()
    PrepareReceptor_11.macroNetwork.unfreeze()

    ## modifying MacroInputNode dynamic ports
    input_Ports_12 = PrepareReceptor_11.macroNetwork.ipNode
    input_Ports_12.outputPorts[1].configure(name='Pdb2pqrWS_CheckFileFormat_value')
    input_Ports_12.outputPorts[2].configure(name='PrepareReceptorWS_PrepareReceptorOpalService_ws_nbcr_net_C')

    ## modifying MacroOutputNode dynamic ports
    output_Ports_13 = PrepareReceptor_11.macroNetwork.opNode
    output_Ports_13.inputPorts[1].configure(singleConnection='auto')
    output_Ports_13.inputPorts[2].configure(singleConnection='auto')
    output_Ports_13.inputPorts[1].configure(name='PrepareReceptorWS_UpdateReceptor_receptor_prepared_obj')
    output_Ports_13.inputPorts[2].configure(name='PrepareReceptorWS_UpdateReceptor_receptor_result')
    PrepareReceptor_11.inputPorts[0].configure(name='Pdb2pqrWS_CheckFileFormat_value')
    PrepareReceptor_11.inputPorts[0].configure(datatype='receptor')
    PrepareReceptor_11.inputPorts[1].configure(name='PrepareReceptorWS_PrepareReceptorOpalService_ws_nbcr_net_C')
    PrepareReceptor_11.inputPorts[1].configure(datatype='boolean')
    ## configure MacroNode input ports
    PrepareReceptor_11.outputPorts[0].configure(name='PrepareReceptorWS_UpdateReceptor_receptor_prepared_obj')
    PrepareReceptor_11.outputPorts[0].configure(datatype='receptor_prepared')
    PrepareReceptor_11.outputPorts[1].configure(name='PrepareReceptorWS_UpdateReceptor_receptor_result')
    PrepareReceptor_11.outputPorts[1].configure(datatype='string')
    ## configure MacroNode output ports
    PrepareReceptor_11.shrink()
    apply(PrepareReceptor_11.configure, (), {'paramPanelImmediate': 1, 'expanded': False})
except:
    print "WARNING: failed to restore PrepareReceptor named PrepareReceptor in network masterNet"
    print_exc()
    PrepareReceptor_11=None

try:
    ## saving node StructureBrowser ##
    from Adt.Input.StructureBrowser import StructureBrowser
    StructureBrowser_29 = StructureBrowser(constrkw={}, name='StructureBrowser', library=Adt)
    masterNet.addNode(StructureBrowser_29,408,35)
    StructureBrowser_29.inputPortByName['receptor_file'].widget.set(r"AutoGrid_0.1_input/2HTY_A.pdbqt", run=False)
    apply(StructureBrowser_29.configure, (), {'paramPanelImmediate': 1})
except:
    print "WARNING: failed to restore StructureBrowser named StructureBrowser in network masterNet"
    print_exc()
    StructureBrowser_29=None

try:
    ## saving node GPFTemplateBrowser ##
    from Adt.Input.GPFTemplateBrowser import GPFTemplateBrowser
    GPFTemplateBrowser_30 = GPFTemplateBrowser(constrkw={}, name='GPFTemplateBrowser', library=Adt)
    masterNet.addNode(GPFTemplateBrowser_30,826,34)
    GPFTemplateBrowser_30.inputPortByName['gpf_template_file'].widget.set(r"AutoGrid_0.1_input/2HTY_A.gpf", run=False)
    apply(GPFTemplateBrowser_30.configure, (), {'paramPanelImmediate': 1})
except:
    print "WARNING: failed to restore GPFTemplateBrowser named GPFTemplateBrowser in network masterNet"
    print_exc()
    GPFTemplateBrowser_30=None

try:
    ## saving node DownloadSaveDir ##
    from WebServices.VisionInterface.WSNodes import DownloadSaveDirNode
    DownloadSaveDir_31 = DownloadSaveDirNode(constrkw={}, name='DownloadSaveDir', library=wslib)
    masterNet.addNode(DownloadSaveDir_31,502,397)
    apply(DownloadSaveDir_31.inputPortByName['url'].configure, (), {'defaultValue': None})
    DownloadSaveDir_31.inputPortByName['url'].rebindWidget()
    DownloadSaveDir_31.inputPortByName['url'].widget.set(r"", run=False)
    DownloadSaveDir_31.inputPortByName['url'].unbindWidget()
    apply(DownloadSaveDir_31.configure, (), {'paramPanelImmediate': 1})
except:
    print "WARNING: failed to restore DownloadSaveDirNode named DownloadSaveDir in network masterNet"
    print_exc()
    DownloadSaveDir_31=None

try:
    ## saving node Output Directory Browser ##
    from Vision.StandardNodes import DirBrowserNE
    Output_Directory_Browser_32 = DirBrowserNE(constrkw={}, name='Output Directory Browser', library=stdlib)
    masterNet.addNode(Output_Directory_Browser_32,905,178)
    Output_Directory_Browser_32.inputPortByName['directory'].widget.set(r"AutoGrid_0.1_output", run=False)
    apply(Output_Directory_Browser_32.configure, (), {'paramPanelImmediate': 1})
except:
    print "WARNING: failed to restore DirBrowserNE named Output Directory Browser in network masterNet"
    print_exc()
    Output_Directory_Browser_32=None

#masterNet.run()
masterNet.freeze()

## saving connections for network AutoGrid_0.1 ##
if PublicServerLigandDB_10 is not None and ComputeGrids_0 is not None:
    try:
        masterNet.connectNodes(
            PublicServerLigandDB_10, ComputeGrids_0, "ligDB", "GetComputeGridsInputs_ligands", blocking=True
            , splitratio=[0.71040241021275885, 0.32432210892569779])
    except:
        print "WARNING: failed to restore connection between PublicServerLigandDB_10 and ComputeGrids_0 in network masterNet"
if PrepareReceptor_11 is not None and ComputeGrids_0 is not None:
    try:
        masterNet.connectNodes(
            PrepareReceptor_11, ComputeGrids_0, "PrepareReceptorWS_UpdateReceptor_receptor_prepared_obj", "GetComputeGridsInputs_receptor_pdbqt", blocking=True
            , splitratio=[0.37771324748037094, 0.29247414694141388])
    except:
        print "WARNING: failed to restore connection between PrepareReceptor_11 and ComputeGrids_0 in network masterNet"
if StructureBrowser_29 is not None and PrepareReceptor_11 is not None:
    try:
        masterNet.connectNodes(
            StructureBrowser_29, PrepareReceptor_11, "receptor_obj", "Pdb2pqrWS_CheckFileFormat_value", blocking=True
            , splitratio=[0.62789739082139784, 0.69687814536090076])
    except:
        print "WARNING: failed to restore connection between StructureBrowser_29 and PrepareReceptor_11 in network masterNet"
if GPFTemplateBrowser_30 is not None and ComputeGrids_0 is not None:
    try:
        masterNet.connectNodes(
            GPFTemplateBrowser_30, ComputeGrids_0, "gpf_template", "GetComputeGridsInputs_gpf_obj", blocking=True
            , splitratio=[0.39300490327970311, 0.4601676294958682])
    except:
        print "WARNING: failed to restore connection between GPFTemplateBrowser_30 and ComputeGrids_0 in network masterNet"
if ComputeGrids_0 is not None and WebBrowser_9 is not None:
    try:
        masterNet.connectNodes(
            ComputeGrids_0, WebBrowser_9, "GetMainURLFromList_newurl", "url", blocking=True
            , splitratio=[0.30896252552604309, 0.21760077157963498])
    except:
        print "WARNING: failed to restore connection between ComputeGrids_0 and WebBrowser_9 in network masterNet"
if ComputeGrids_0 is not None and DownloadSaveDir_31 is not None:
    try:
        masterNet.connectNodes(
            ComputeGrids_0, DownloadSaveDir_31, "GetMainURLFromList_newurl", "url", blocking=True
            , splitratio=[0.66446304698655212, 0.30837982537508668])
    except:
        print "WARNING: failed to restore connection between ComputeGrids_0 and DownloadSaveDir_31 in network masterNet"
if Output_Directory_Browser_32 is not None and DownloadSaveDir_31 is not None:
    try:
        masterNet.connectNodes(
            Output_Directory_Browser_32, DownloadSaveDir_31, "directory", "path", blocking=True
            , splitratio=[0.34510585020439977, 0.73683619505012499])
    except:
        print "WARNING: failed to restore connection between Output_Directory_Browser_32 and DownloadSaveDir_31 in network masterNet"
masterNet.runOnNewData.value = False

if __name__=='__main__':
    from sys import argv
    lNodePortValues = []
    if (len(argv) > 1) and argv[1].startswith('-'):
        lArgIndex = 2
    else:
        lArgIndex = 1
    while lArgIndex < len(argv) and argv[lArgIndex][-3:]!='.py':
        lNodePortValues.append(argv[lArgIndex])
        lArgIndex += 1
    masterNet.setNodePortValues(lNodePortValues)
    if '--help' in argv or '-h' in argv: # show help
        masterNet.helpForNetworkAsApplication()
    elif '-w' in argv: # run without Vision and exit
         # create communicator
        from NetworkEditor.net import Communicator
        masterNet.communicator = Communicator(masterNet)
        print 'Communicator listening on port:', masterNet.communicator.port

        import socket
        f = open(argv[0]+'.sock', 'w')
        f.write("%s %i"%(socket.gethostbyname(socket.gethostname()),
                         masterNet.communicator.port))
        f.close()

        masterNet.run()

    else: # stand alone application while vision is hidden
        if '-e' in argv: # run and exit
            masterNet.run()
        elif '-r' in argv or len(masterNet.userPanels) == 0: # no user panel => run
            masterNet.run()
            mainLoopVisionToRunNetworkAsApplication(masterNet.editor)
        else: # user panel
            mainLoopVisionToRunNetworkAsApplication(masterNet.editor)




#       
#           AutoDock | Raccoon2
#
#       Copyright 2013, Stefano Forli
#          Molecular Graphics Lab
#  
#     The Scripps Research Institute 
#           _  
#          (,)  T  h e
#         _/
#        (.)    S  c r i p p s
#          \_
#          (,)  R  e s e a r c h
#         ./  
#        ( )    I  n s t i t u t e
#         '
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import re
import DebugTools
from itertools import groupby
import CADD.Raccoon2.HelperFunctionsN3P as hf

class FilterEngine(DebugTools.DebugObj):
    """ engine to filter results accordingly to
        selected engine and active filters
    """
    def __init__(self, app, debug=False):
        DebugTools.DebugObj.__init__(self, debug)
        self.app = app
                                                      # float(min) < float(max) always!
        self.settings = {  'filters': { 'energy' : { 'values' :[0, 1],
                                                     'cb'     : None,
                                                     'function' : self.energyFilter,
                                                   },

                                        'leff'   : { 'values' : [0, 1],
                                                     'cb'     : None,
                                                     'function' : self.leffFilter,
                                                   },

                                        'interactions' : { 'function': self.interactFilter,
                                                      'pattern' : [],
                                                    }
                                       # NOTES 
                                       # a filter is essentially defined 
                                       # by a function and a series of settings 
                                       # this function will know how to read
                                       # 
                                       # filtername : { 'function' : function,
                                       #                'filtersettings' : data
                                       #               }
                                      },

                           'pose':  'first', # first, any         
                           'order': [ 'energy', 'leff', 'interactions'],
                        }
        self.STOP = False

    def setFilters(self, filters={}):
        """ update filter settings"""
        for f, values in filters.items():
            self.settings['filters'][f].update(values)
        #print "\n=====\n", self.settings, "\n=====\n"

    def doFilter(self, cb = None):
        """
            this is the actual filter engine.
            it calls all the functions that are defined in the 
            perform the actual filtering
            countOnly=True: return the number of items passing
            cb is a function callback

        self.app.results = { 'resLogName' : { 'lig1' : 
                                        { 'data': 
                                            { 'energy': xxx, 
                                             }}
                                        { 'accepted':  [ 0, 2, 3]} # poses passing the filter
                                    }}
        """
        accepted = {'total' : [] }
        #total = []
        count = 0
        for resultName, data in self.app.results.items():
            result = data['results']
            for ligName, properties in result.items():
                ligId = (resultName, ligName)
                if self.STOP:
                    self.STOP = False
                    return False
                count += 1
                # reset ligand poses that have been accepted
                properties['accepted'] = range(len(properties['data']))
                # filter ligand with functions defined in self.settings
                #for filtName, filtData in self.settings['filters'].items():
                for filtName in self.settings['order']:
                    filtData = self.settings['filters'][filtName]
                    self.dprint("Filter [%s]: %s" % (filtName, filtData) )
                    if not filtName in accepted.keys():
                        accepted[filtName] = []
                    filtFunction = filtData['function'] 
                    if properties['accepted']:
                        properties['accepted'] = filtFunction(properties)
                    if len(properties['accepted']):
                        accepted[filtName].append(ligId)
                if len(properties['accepted']):
                    accepted['total'].append(ligId)
        if cb: cb(len(accepted['total']))
        return accepted


    def energyFilter(self, ligProperties):
        """ energy filter
            check if the energy of the ligand defined in
            ligProperties matches the requirements
            energy requirement and pose number are obtained
            from settings
        """
        # callbacks # NOTE not used for now
        e_cb =  self.settings['filters']['energy']['cb']
        # filter values
        emin, emax = self.settings['filters']['energy']['values']
        # which pose is requested
        opt = self.settings['pose']
        # gather ligand info and poses to scan
        ligInfo = ligProperties['data']
        ligAcceptedPoses = ligProperties['accepted']
        if opt == 'first':
            poses = [0]
        elif opt == 'any':
            poses = range(len( ligProperties.keys())) 
        passed = []
        for p in ligProperties['accepted']:
            if p in poses:
                e = ligInfo[p]['energy']
                if emin <= e <= emax:
                    #e_cb()
                    passed.append(p)
        return passed
    
    def leffFilter(self, ligProperties):
        """ ligand efficiency filter"""
        # callbacks # NOTE not used for now
        le_cb =  self.settings['filters']['leff']['cb']
        # filter values
        lemin, lemax = self.settings['filters']['leff']['values']
        # which pose is requested
        opt = self.settings['pose']
        # gather ligand info and poses to scan
        ligInfo = ligProperties['data']
        ligAcceptedPoses = ligProperties['accepted']
        if opt == 'first':
            poses = [0]
        elif opt == 'any':
            poses = range(len( ligProperties.keys())) 
        passed = []
        for p in ligProperties['accepted']:
            if p in poses:
                le = ligInfo[p]['leff']
                if lemin <= le <= lemax:
                    #e_cb()
                    passed.append(p)
        return passed


    def interactFilter(self, ligProperties):
        """ interaction filter """
        settings = self.settings['filters']['interactions']
        requested = settings['pattern']
        mode = settings['mode'] # any, all
        poses = range(len( ligProperties.keys())) 
        passed = []
        opt = self.settings['pose']
        if opt == 'first':
            poses = [0]
        # compile requested interaction patterns
        requestedInteractions = []
        for reqType in sorted(requested.keys()):
            for reqPatt in requested[reqType]:
                requestedInteractions.append( (reqType, reqPatt[0], reqPatt[1]) )
        if len(requestedInteractions) == 0:
            return poses
        for p in ligProperties['accepted']:
            if p in poses:
                interactionPool = ligProperties['data'][p]['interactions']
                satisfied = False
                for rtype, rpatt, rwanted in requestedInteractions:
                    pool = self.getUsefulInteractions(rtype, interactionPool)
                    satisfied = self.matchInteraction(rpatt, pool, rwanted)
                    if satisfied and (mode == 'any'): # one success is enough
                        break
                    elif not satisfied and (mode == 'all'): # one failure is too much
                        break
                if satisfied:
                    passed.append(p)
        return passed
 
    def getUsefulInteractions(self, rtype, interactionPool):
        """ extract useful interactions for the requested
            interaction type
        """
        pool = []
        try:
            if rtype == 'hb':   # any hb interaction, rec atom
                if 'hba' in interactionPool.keys():
                    pool += interactionPool['hba']
                if 'hbd' in interactionPool.keys():
                    pool += interactionPool['hbd']
                pool = [ x[1] for x in pool ]
            elif rtype == 'hba':   # hba, rec atom
                pool = [x[1] for x in interactionPool['hba'] ]
            elif rtype == 'hbd':   # hbd, rec atom
                pool = [x[1] for x in interactionPool['hbd'] ]
            elif rtype == 'pi':    # any pi interaction, rec atom
                if 'ppi' in interactionPool.keys():
                    pool += interactionPool['ppi']
                if 'tpi' in interactionPool.keys():
                    pool += interactionPool['tpi']
                pool = [x[0] for x in pool ]
            elif rtype == 'ppi':   # pi interaction, rec atom
                pool = [x[0] for x in interactionPool['ppi'] ]
            elif rtype == 'tpi':   # t-pi interaction, rec atom
                pool = [x[0] for x in interactionPool['tpi'] ]
            elif rtype == 'metal': # metal coordination, rec atom
                pool = [x[1] for x in interactionPool['metal'] ]
            else:  # vdw
                pool = interactionPool[rtype]
        except KeyError:
            pass
        return pool


    def matchInteraction(self, pattern, pool, wanted = True):
        """ use RE to match interactions 
            pattern : requested
            pool : pool of interactions where pattern
                              is going to be searched
            wanted: specify if the interaction is wanted or not
        """
        # B:ASN249:ND2
        pattern = pattern.replace("*", ".*")
        pattern = pattern.replace("?", ".?")
        for entry in pool:
            if re.search(pattern, entry):
                return True == wanted
        return False == wanted




